/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : common.h
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the common configuration macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1              21/01/2022          Rajesh
   0.0.1.0              07/06/2022          Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

#ifndef _COMMON_H_
#define	_COMMON_H_

/* Includes ------------------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "main.h"
#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os.h"
#include "string.h"
#include <stdbool.h>
#include "stdio.h"
#include "queue.h"
#include "timers.h"
#include "semphr.h"
#include "Ps_Typedef.h"
#include "../../../../Core/Maximatecc/user_modify_files/Ai280_Config.h"
#include "stm32h7xx_hal_i2c.h"
#include "../../Maximatecc/Inc/Data_Layer.h"
#include "Analog_Handler.h"
#include "Conf_Platformservice.h"
#include "Log.h"
#include "Memory_Management.h"
#include "Platform_Service.h"
#include "../../../../Core/Maximatecc/user_modify_files/EE_PH_DB.h"

#if (SDK_SERVICE_KEYPAD == PS_ENABLE)
#include "Keypad_Platformservice.h"
#endif

#if (SDK_SERVICE_RTC == PS_ENABLE)
#include "Rtc_Platformservice.h"
#endif

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)
#include "Digitalio_Platformservice.h"
#endif

#if (SDK_SERVICE_LED == PS_ENABLE)
#include "Led_Platformservice.h"
#include "../../../Drivers/Maximatecc/inc/Led.h"
#endif

#if ((SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE) || \
		(SDK_SERVICE_CFG_INPUT == PS_ENABLE))
#include "../../Drivers/Maximatecc/inc/input_control.h"
#include "../../Drivers/Maximatecc/inc/Digitalio.h"
#endif

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)
#include "Adc_Init.h"
#include "Frequency_Timer_Init.h"
#include "ConfigurableInput_platformservice.h"
#include "../../../Drivers/Maximatecc/inc/Adc.h"
#include "../../../Drivers/Maximatecc/inc/ConfigurableInput.h"
#include "../../../Drivers/Maximatecc/inc/Input_Frequency.h"
#endif

#if (SDK_SERVICE_LCD == PS_ENABLE)
#include "Lcd_platformservice.h"
#endif

#if (SDK_SERVICE_PM == PS_ENABLE)
#include "PowerMode_Platformservice.h"
#include "../../../Drivers/Maximatecc/inc/Power_Mode.h"
#endif

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)
#include "PowerMonitor_Platformservice.h"
#endif

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
#include "Watchdog_Platformservice.h"
#endif

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
#include "EEPROM_Platformservice.h"
#include "../../Drivers/Maximatecc/inc/eeprom_spi.h"
#endif

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)
#include "SWTIMER_Platformservice.h"
#endif

#if (SDK_SERVICE_BLE == PS_ENABLE)
#include "TerminalCommand.h"
#endif

#if (SDK_SERVICE_BLE == PS_ENABLE)
#include "Bluetooth_Platformservice.h"
#include "../../Drivers/Maximatecc/inc/bluetooth.h"
#endif

#if (SDK_SERVICE_QSPI == PS_ENABLE)
#include "Qspi.h"
#endif

#if (SDK_SERVICE_J1939 == PS_ENABLE)
#include "J1939.h"
#endif

#if (SDK_SERVICE_FDCAN == PS_ENABLE)
#include "Fdcan_Platformservice.h"
#endif

#include "Sdk_Api.h"

typedef uint8_t (*ss_cb)(uint8_t);
/* USER CODE END Includes */

/********************************  CONSTANTS  *********************************/
/* USER CODE BEGIN Private defines */
#define FULL 	1
#define NO_FULL 2

#ifndef TRUE
#define   TRUE  (!0)
#endif

#ifndef FALSE
#define   FALSE 0
#endif

#ifndef SUCCESS
#define   SUCCESS  1
#endif

#ifndef FAILURE
#define   FAILURE  0
#endif

#ifndef MEM_FAILURE
#define   MEM_FAILURE  2
#endif

#define CLEAR_BITS_ONENTRY	0x00
#define ULONG_MAX			0xffffffff
#define int8_t  			signed char

#define MEM_QPKTSIZE    	(20)
#define MEM_INPUT_BUFF    	(50)
#define MEM_QBUFFSIZE   	(MEM_QPKTSIZE * MEM_INPUT_BUFF)
#define MAX_RETRY_Q			3
/* USER CODE END Private defines */

#define NUMBER_SDK_SERVICE  20

/* USER CODE BEGIN PTD */
/*!
 * Private structure for the Platform Service task
 */
typedef struct __pvt_variable{
	uint8_t            		membuf[MEM_QBUFFSIZE];
	QueueHandle_t	   		membufq;
	TimerHandle_t      		xTimers[PS_MAX_TIMER];
	TaskHandle_t       		Conf_task;
	uint8_t            		isPSUP;
	//LTDC_HandleTypeDef *hltdc;
	uint8_t            		start_com;

#if ((SDK_SERVICE_LED == PS_ENABLE) ||        \
		(SDK_SERVICE_CFG_INPUT == PS_ENABLE))
	I2C_HandleTypeDef  		*hi2c;
	SemaphoreHandle_t  		i2c_mutex;
#endif

#if ((SDK_SERVICE_EEPROM == PS_ENABLE) || \
		(SDK_SERVICE_BLE == PS_ENABLE))
	SPI_HandleTypeDef  		*hspi1;
	SPI_HandleTypeDef  		*hspi2;
	SemaphoreHandle_t  		spi_mutex;
#endif

#if (SDK_SERVICE_KEYPAD == PS_ENABLE)
	QueueHandle_t      		kpdtxq;
	QueueHandle_t      		kpdrxq;
	TaskHandle_t       		kpd_task;
#endif

#if (SDK_SERVICE_RTC == PS_ENABLE)
	rtc_ps			   		rtc_ps_data;
	QueueHandle_t      		rtctxq;
	QueueHandle_t      		rtcrxq;
	TaskHandle_t       		rtc_task;
	RTC_HandleTypeDef  		*hrtc;
	RTC_TimeTypeDef    		sTime;
	RTC_DateTypeDef    		sDate;
	RTC_AlarmTypeDef   		sAlarm;
#endif

#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)
	QueueHandle_t      		dotxq;
	QueueHandle_t      		dorxq;
	TaskHandle_t       		do_task;
	uint8_t            		doscfg[MAX_NUM_OF_DIO];
	uint8_t                 dostate[MAX_NUM_OF_DIO];
#endif

#if (SDK_SERVICE_LED == PS_ENABLE)
	QueueHandle_t      		ledtxq;
	QueueHandle_t      		ledrxq;
	TaskHandle_t       		led_task;
#endif

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)
	ADC_HandleTypeDef  		*hadc1;
	ADC_HandleTypeDef  		*hadc3;
	TIM_HandleTypeDef  		*htim2;
	TIM_HandleTypeDef  		*htim3;
	TIM_HandleTypeDef  		*htim13;
	TIM_HandleTypeDef  		*htim14;
	QueueHandle_t      		citxq;
	QueueHandle_t      		cirxq;
	TaskHandle_t       		ci_task;
	uint16_t           		channelA_value[MAX_NUM_CFG_INPUT];
	uint16_t           		channelB_value[MAX_NUM_CFG_INPUT];
#endif

#if (SDK_SERVICE_LCD == PS_ENABLE)
	QueueHandle_t      		lcdtxq;
	QueueHandle_t      		lcdrxq;
	TaskHandle_t       		lcd_task;
	TIM_HandleTypeDef  		*htim1;
#endif

#if (SDK_SERVICE_PM == PS_ENABLE)
	QueueHandle_t      		pmtxq;
	QueueHandle_t      		pmrxq;
	TaskHandle_t       		pm_task;
	PM_Callbacks       		PM_CB[MAX_NUMBER_PS];
    uint8_t            		num_cb;
#endif

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)
	QueueHandle_t      		pmonitortxq;
	QueueHandle_t      		pmonitorrxq;
	TaskHandle_t       		pmonitor_task;
#endif

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
	uint8_t            		WatchDog_reset;
	IWDG_HandleTypeDef  	*iwdg;
	QueueHandle_t      		wdtxq;
	QueueHandle_t      		wdrxq;
	TaskHandle_t       		wd_task;
#endif

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
	CRC_HandleTypeDef  		*hcrc;
	QueueHandle_t      		eetxq;
	QueueHandle_t      		eerxq;
	TaskHandle_t       		ee_task;
#endif

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)
	QueueHandle_t       	swttxq;
	QueueHandle_t       	swtrxq;
	TaskHandle_t        	swt_task;
	TIM_HandleTypeDef   	*htim4;
	TIM_HandleTypeDef   	*htim7;
	TIM_HandleTypeDef   	*htim8;
	TIM_HandleTypeDef   	*htim12;
	TIM_HandleTypeDef   	*htim17;
	TIM_HandleTypeDef   	*htim16;
	uint8_t 				sw1_mode;
	uint16_t 				sw1_timeout;
	uint32_t 				sw1_count;
	uint8_t 				sw2_mode;
	uint16_t 				sw2_timeout;
	uint32_t 				sw2_count;
	uint8_t 				sw3_mode;
	uint16_t 				sw3_timeout;
	uint32_t 				sw3_count;
	uint8_t 				sw4_mode;
	uint16_t 				sw4_timeout;
	uint32_t 				sw4_count;
	uint8_t 				sw5_mode;
	uint16_t 				sw5_timeout;
	uint32_t 				sw5_count;
	uint8_t 				sw6_mode;
	uint16_t 				sw6_timeout;
	uint32_t 				sw6_count;
#endif

#if (SDK_SERVICE_BLE == PS_ENABLE)
	QueueHandle_t       	bletxq;
	QueueHandle_t       	blerxq;
	TaskHandle_t        	ble_task;
	uint8_t             	blerxpkt[MAX_BLE_RX_LEN];
	uint8_t             	bletrxpkt[MAX_BLE_RX_LEN];
	uint8_t             	bletxpkt[MAX_BLE_TX_LEN];
	uint8_t             	blerxindex;
	uint8_t             	blestart_com;
	uint8_t             	bletx_status;
#endif

#if (SDK_SERVICE_QSPI == PS_ENABLE)
	QSPI_HandleTypeDef    	*hqspi;
#endif

#if (SDK_SERVICE_FDCAN == PS_ENABLE)
	QueueHandle_t         	fdtxq;
	QueueHandle_t         	fdrxq;
	TaskHandle_t          	fdcan_task;
	TaskHandle_t          	fdcanrx1_task;
	FDCAN_HandleTypeDef   	*hfdcan1;
	FDCAN_TxHeaderTypeDef 	txHeader;
	FDCAN_RxHeaderTypeDef 	rx1Header;
	FDCAN_RxHeaderTypeDef 	rx2Header;
	FDCAN_FilterTypeDef   	sFilterConfig;

	/* CAN QBuffer */
	uint8_t               	can1membuf[MEM_QBUFFSIZE];
	QueueHandle_t	   	  	memcan1bufq;
	QueueHandle_t	   	  	memcan1rxbufq;

#endif

#if (SDK_SERVICE_J1939 == PS_ENABLE)
	TaskHandle_t       		j1939_task;
	TaskHandle_t       		j1939_ctrltask;
	QueueHandle_t      		jctrlrxq;
	QueueHandle_t      		jctrltxq;
	uint16_t           		jcan1_dest_address;
#if (EMTOS_J1939_DM1 == PS_ENABLE)
	uint8_t            		send_j1939_can1_dm1;
#endif

#if (EMTOS_J1939_DM2 == PS_ENABLE)
	uint8_t            		send_j1939_can1_dm2;
#endif

#if (EMTOS_J1939_DM3 == PS_ENABLE)
	uint8_t            		send_j1939_can1_dm3;
#endif
#endif

#if (THROUGH_PUT_SERVICE == PS_ENABLE)
	TaskHandle_t 			tp_task;
#endif

}pvt_variable;
/* USER CODE END PTD */

#endif	//_COMMON_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
