/****************************************************************************
              Copyright (c) 2021 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : ConfigurableInput_platformservice.h
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the ConfigurableInput platform service declarations

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _CONFIGURABLEINPUT_PLATFORM_SERVICE_H_
#define	_CONFIGURABLEINPUT_PLATFORM_SERVICE_H_


/*!
 * ConfigurableInput TxQ and RxQ size for
 * receiving data from SDK API.
 */
#define CI_RX_TX_MAX_Q_SIZE 				10

/*!
 * Minimum and Maximum Range of sample to get in a second
 */
#define MIN_NUM_SAMPLE         				1
#define MAX_NUM_SAMPLE         				255

/*!
 * Configurable Input Timeout value
 */
#define PS_CI_TIMEOUT						100

/*!
 * Default number of samples
 */
#define DEFAULT_NUM_SAMPLES    				10

/*!
 * Min and Max number for the Configurable Input
 */
#define MIN_NUM_CFG_INPUT      				1
#define MAX_NUM_CFG_INPUT      				5

/*!
 * Minimum and Maximum Range of voltage
 */
#define MIN_VOLTAGE_RANGE         			0
#define MAX_VOLTAGE_RANGE         			32000

/*!
 * Minimum and Maximum Range of resistance
 */
#define MIN_RESISTANCE_RANGE         		0
#define MAX_RESISTANCE_RANGE         		10000

/*!
 * Minimum and Maximum Range of frequency
 */
#define MIN_FREQUENCY_RANGE         		0
#define MAX_FREQUENCY_RANGE         		200000

/*!
 * Minimum and Maximum Range of current
 */
#define MIN_CURRENT_RANGE	         		0
#define MAX_CURRENT_RANGE          			200000

/*!
 * ANALOG MAPPING
 */
#define ANLG_MAP_01_A    					2
#define ANLG_MAP_01_B    					12
#define ANLG_MAP_02_A    					3
#define ANLG_MAP_02_B    					11
#define ANLG_MAP_03_A    					4
#define ANLG_MAP_03_B    					6
#define ANLG_MAP_04_A    					7
#define ANLG_MAP_04_B    					5
#define ANLG_MAP_05_A    					9
#define ANLG_MAP_05_B    					0
#define ANLG_MAP_06_A    					10
#define ANLG_MAP_06_B    					1

/*!
 * Configuration Input Index
 */
#define CONF_AI1    						0
#define CONF_AI2    						1
#define CONF_AI3    						2
#define CONF_AI4    						3
#define CONF_AI5    						4
#define CONF_AI6    						5
#define CONF_NONE   						6

/*!
 * Configurable Platform Service Task Size
 */
#define CI_TASK_SIZE	 					512

/* !
 * Configurable SDK Field Index
 */
#define CI_FUNC_INDEX             			0
#define CI_RESULT_INDEX           			0
#define CI_SET_ID_INDEX			  			1
#define CI_GET_ID_INDEX			  			1
#define CI_TYPE_INDEX			  			2
#define CI_VALUE_INDEX			  			3

/*!
 * Configurable Input parameter configuration structure
 */
typedef struct __ci_conf_param
{
	uint8_t db_type_index;    /* DB type index                   */
	uint8_t db_sample_index;  /* DB Number of sample Index       */
    uint8_t  type;             /* Input Type                      */
    uint8_t  num_sample;       /* Number of the sample per second */
    uint8_t  elapse_count;     /* Elapsed time count for the sample per second */
    uint8_t  AChanADC;         /* ADC connected channel           */
    uint8_t  BChanADC;         /* ADC connected channel           */
    uint8_t  Reserved;         /* ADC connected channel           */
}ci_conf_param;

/*!
 * ConfigurableInput SDK Message
 */
typedef enum __CI_MSG
{
	CI_SET,
	CI_GET,
	CI_SET_INPUT_TYPE,
	CI_SET_NUM_SAMPLE,
	CI_GET_INPUT_TYPE,
	CI_GET_NUM_SAMPLE,
}CI_MSG;

/*!
 * Configurable Input Events
 */
typedef enum __CI_Event
{
	CI_TMR_EVENT,
	CI_MSG_EVENT,
}CI_Event;

/*!
 * Configurable Input configuration
 */
typedef enum __CI_INPUT
{
    START_CI_INPUT,
	CI_INPUT_FREQUENCY,
	CI_INPUT_CURRENT,
	CI_INPUT_VOLTAGE_LOW,
	CI_INPUT_RESISTANCE,
	CI_DIGITAL_STG,
	CI_DIGITAL_STB,
	CI_INPUT_VOLTAGE_HIGH,
	CI_INPUT_VOLTAGE_LOW_6V,
	CI_INPUT_OFF,
	END_CI_INPUT,
}CI_INPUT;

/*!
 * Function declaration
 */
void CI_DB_Init(void);


#endif	//_CONFIGURABLEINPUT_PLATFORM_SERVICE_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
