/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Adc_Init.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the ADC platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/common.h"
#include "../Inc/Adc_Init.h"
/* USER CODE END Includes */

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

extern ci_conf_param ci_conf[MAX_NUM_CFG_INPUT];
extern pvt_variable pv;

extern DMA_HandleTypeDef hdma_adc1;
extern DMA_HandleTypeDef hdma_adc3;

/**
  * @brief ADC Initialization Function
  * @param None
  * @retval None
  */
void ADC_Initialization(void)
{
	ADC1_Init();
	ADC3_Init();
	ADC_start();
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
void ADC1_Init(void)
{
	/* USER CODE BEGIN ADC1_Init 0 */
	HAL_ADC_DeInit(pv.hadc1);
	/* USER CODE END ADC1_Init 0 */

	ADC_MultiModeTypeDef multimode = {0};
	ADC_ChannelConfTypeDef sConfig = {0};

	/* USER CODE BEGIN ADC1_Init 1 */

	/* USER CODE END ADC1_Init 1 */
	/** Common config */
	pv.hadc1->Instance = ADC1;
	pv.hadc1->Init.ClockPrescaler = ADC_CLOCK_ASYNC_DIV6;
	pv.hadc1->Init.Resolution = ADC_RESOLUTION_12B;
	pv.hadc1->Init.ScanConvMode = ADC_SCAN_ENABLE;
	pv.hadc1->Init.EOCSelection = ADC_EOC_SINGLE_CONV;
	pv.hadc1->Init.LowPowerAutoWait = DISABLE;
	pv.hadc1->Init.ContinuousConvMode = ENABLE;
	pv.hadc1->Init.NbrOfConversion = 9;
	pv.hadc1->Init.DiscontinuousConvMode = DISABLE;
	pv.hadc1->Init.ExternalTrigConv = ADC_SOFTWARE_START;
	pv.hadc1->Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	pv.hadc1->Init.ConversionDataManagement = ADC_CONVERSIONDATA_DMA_CIRCULAR;
	pv.hadc1->Init.Overrun = ADC_OVR_DATA_OVERWRITTEN;
	pv.hadc1->Init.LeftBitShift = ADC_LEFTBITSHIFT_NONE;
	pv.hadc1->Init.OversamplingMode = DISABLE;
	if (HAL_ADC_Init(pv.hadc1) != HAL_OK)
	{
		Error_Handler();
	}

	/* Deinitialize the Default IOC Configuration */
	HAL_ADC_MspDeInit(pv.hadc1);

	/* Reinitialize the IOC Configuration */
	HAL_ADC_MspInit_2(pv.hadc1);

	/** Configure the ADC multi-mode */
	multimode.Mode = ADC_MODE_INDEPENDENT;
	if (HAL_ADCEx_MultiModeConfigChannel(pv.hadc1, &multimode) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_7;
	sConfig.Rank = ADC_REGULAR_RANK_1;
	sConfig.SamplingTime = ADC_SAMPLETIME_387CYCLES_5;
	sConfig.SingleDiff = ADC_SINGLE_ENDED;
	sConfig.OffsetNumber = ADC_OFFSET_NONE;
	sConfig.Offset = 0;
	sConfig.OffsetSignedSaturation = DISABLE;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_9;
	sConfig.Rank = ADC_REGULAR_RANK_2;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_10;
	sConfig.Rank = ADC_REGULAR_RANK_3;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_8;
	sConfig.Rank = ADC_REGULAR_RANK_4;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_15;
	sConfig.Rank = ADC_REGULAR_RANK_5;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_16;
	sConfig.Rank = ADC_REGULAR_RANK_6;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_17;
	sConfig.Rank = ADC_REGULAR_RANK_7;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_19;
	sConfig.Rank = ADC_REGULAR_RANK_8;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_5;
	sConfig.Rank = ADC_REGULAR_RANK_9;
	if (HAL_ADC_ConfigChannel(pv.hadc1, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/* USER CODE BEGIN ADC1_Init 2 */
	HAL_ADC_Stop_DMA(pv.hadc1);
	/* USER CODE END ADC1_Init 2 */
}

/**
  * @brief ADC3 Initialization Function
  * @param None
  * @retval None
  */
void ADC3_Init(void)
{
	/* USER CODE BEGIN ADC3_Init 0 */
	HAL_ADC_DeInit(pv.hadc3);
	/* USER CODE END ADC3_Init 0 */
	ADC_ChannelConfTypeDef sConfig = {0};

	/* USER CODE BEGIN ADC3_Init 1 */

	/* USER CODE END ADC3_Init 1 */
	pv.hadc3->Instance = ADC3;
	pv.hadc3->Init.ClockPrescaler = ADC_CLOCK_ASYNC_DIV6;
	pv.hadc3->Init.Resolution = ADC_RESOLUTION_12B;
	pv.hadc3->Init.ScanConvMode = ADC_SCAN_ENABLE;
	pv.hadc3->Init.EOCSelection = ADC_EOC_SINGLE_CONV;
	pv.hadc3->Init.LowPowerAutoWait = DISABLE;
	pv.hadc3->Init.ContinuousConvMode = ENABLE;
	pv.hadc3->Init.NbrOfConversion = 6;
	pv.hadc3->Init.DiscontinuousConvMode = DISABLE;
	pv.hadc3->Init.ExternalTrigConv = ADC_SOFTWARE_START;
	pv.hadc3->Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
	pv.hadc3->Init.ConversionDataManagement = ADC_CONVERSIONDATA_DMA_CIRCULAR;
	pv.hadc3->Init.Overrun = ADC_OVR_DATA_OVERWRITTEN;
	pv.hadc3->Init.LeftBitShift = ADC_LEFTBITSHIFT_NONE;
	pv.hadc3->Init.OversamplingMode = DISABLE;
	if (HAL_ADC_Init(pv.hadc3) != HAL_OK)
	{
		Error_Handler();
	}

	/* Deinitialize the Default IOC Configuration */
	HAL_ADC_MspDeInit(pv.hadc3);

	/* Reinitialize the IOC Configuration */
	HAL_ADC_MspInit_2(pv.hadc3);

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_0;
	sConfig.Rank = ADC_REGULAR_RANK_1;
	sConfig.SamplingTime = ADC_SAMPLETIME_387CYCLES_5;
	sConfig.SingleDiff = ADC_SINGLE_ENDED;
	sConfig.OffsetNumber = ADC_OFFSET_NONE;
	sConfig.Offset = 0;
	sConfig.OffsetSignedSaturation = DISABLE;
	if (HAL_ADC_ConfigChannel(pv.hadc3, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_1;
	sConfig.Rank = ADC_REGULAR_RANK_2;
	if (HAL_ADC_ConfigChannel(pv.hadc3, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_2;
	sConfig.Rank = ADC_REGULAR_RANK_3;
	if (HAL_ADC_ConfigChannel(pv.hadc3, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_7;
	sConfig.Rank = ADC_REGULAR_RANK_4;
	if (HAL_ADC_ConfigChannel(pv.hadc3, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_TEMPSENSOR;
	sConfig.Rank = ADC_REGULAR_RANK_5;
	if (HAL_ADC_ConfigChannel(pv.hadc3, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/** Configure Regular Channel */
	sConfig.Channel = ADC_CHANNEL_VREFINT;
	sConfig.Rank = ADC_REGULAR_RANK_6;
	if (HAL_ADC_ConfigChannel(pv.hadc3, &sConfig) != HAL_OK)
	{
		Error_Handler();
	}

	/* USER CODE BEGIN ADC3_Init 2 */
	HAL_ADC_Stop_DMA(pv.hadc3);
	/* USER CODE END ADC3_Init 2 */
}

/**
* @brief ADC MSP Initialization
* This function configures the hardware resources used in this example
* @param hadc: ADC handle pointer
* @retval None
*/
void HAL_ADC_MspInit_2(ADC_HandleTypeDef* hadc)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	RCC_PeriphCLKInitTypeDef PeriphClkInitStruct = {0};
	if(hadc->Instance==ADC1)
	{
	    PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_ADC;
	    PeriphClkInitStruct.PLL2.PLL2M = 5;
	    PeriphClkInitStruct.PLL2.PLL2N = 192;
	    PeriphClkInitStruct.PLL2.PLL2P = 40;
	    PeriphClkInitStruct.PLL2.PLL2Q = 2;
	    PeriphClkInitStruct.PLL2.PLL2R = 2;
	    PeriphClkInitStruct.PLL2.PLL2RGE = RCC_PLL2VCIRANGE_2;
	    PeriphClkInitStruct.PLL2.PLL2VCOSEL = RCC_PLL2VCOWIDE;
	    PeriphClkInitStruct.PLL2.PLL2FRACN = 0;
	    PeriphClkInitStruct.AdcClockSelection = RCC_ADCCLKSOURCE_PLL2;
	    if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
	    {
	      Error_Handler();
	    }

		/* USER CODE BEGIN ADC1_MspInit 0 */

		/* USER CODE END ADC1_MspInit 0 */
		/* Peripheral clock enable */
		__HAL_RCC_ADC12_CLK_ENABLE();

		__HAL_RCC_GPIOC_CLK_ENABLE();
		__HAL_RCC_GPIOA_CLK_ENABLE();
		__HAL_RCC_GPIOB_CLK_ENABLE();
		/**ADC1 GPIO Configuration
		 * PC0     ------> ADC1_IN10
		 * PC5     ------> ADC1_IN4
		 * PA0/WKUP     ------> ADC1_IN0
		 * PA1     ------> ADC1_IN1
		 * PA3     ------> ADC1_IN3
		 * PA5     ------> ADC1_IN5
		 * PA7     ------> ADC1_IN7
		 * PB0     ------> ADC1_IN8
		 * PB1     ------> ADC1_IN9
		 * */
		if(ci_conf[0].type != INPUT_FREQUENCY)//AI 1 adc3 tim 13 port F pin 8 and adc1 port c pin 0
		{
			GPIO_InitStruct.Pin = GPIO_PIN_0;         // port C pin 0
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);
		}

		if(ci_conf[1].type != INPUT_FREQUENCY)//AI 2 adc3 tim 14 port F pin 9 and adc1 port c pin 1
		{
			GPIO_InitStruct.Pin = GPIO_PIN_5;         // port C pin 1
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);
		}

		if(ci_conf[2].type != INPUT_FREQUENCY)//AI 3 adc1 tim 2 chn 2 port A pin 1
		{
			GPIO_InitStruct.Pin = GPIO_PIN_1;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

			GPIO_InitStruct.Pin = GPIO_PIN_3;         // port A pin 3
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
		}

		if(ci_conf[3].type != INPUT_FREQUENCY)//AI 4 adc1 tim 2 chn 1 port A pin 0
		{
			GPIO_InitStruct.Pin = GPIO_PIN_0;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

			GPIO_InitStruct.Pin = GPIO_PIN_5;           //port A pin 5
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
		}

		if(ci_conf[4].type != INPUT_FREQUENCY)//AI 5 adc1 tim 3 chn 2 port A pin 7
		{
			GPIO_InitStruct.Pin = GPIO_PIN_7;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
		}

		GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1;
		GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
		GPIO_InitStruct.Pull = GPIO_NOPULL;
		HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

		HAL_DMA_DeInit(&hdma_adc1);

		/* USER CODE BEGIN ADC1_MspInit 1 */
	    /* ADC1 DMA Init */
	    /* ADC1 Init */
	    hdma_adc1.Instance = DMA2_Stream4;
	    hdma_adc1.Init.Request = DMA_REQUEST_ADC1;
	    hdma_adc1.Init.Direction = DMA_PERIPH_TO_MEMORY;
	    hdma_adc1.Init.PeriphInc = DMA_PINC_DISABLE;
	    hdma_adc1.Init.MemInc = DMA_MINC_ENABLE;
	    hdma_adc1.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;
	    hdma_adc1.Init.MemDataAlignment = DMA_MDATAALIGN_WORD;
	    hdma_adc1.Init.Mode = DMA_CIRCULAR;
	    hdma_adc1.Init.Priority = DMA_PRIORITY_VERY_HIGH;
	    hdma_adc1.Init.FIFOMode = DMA_FIFOMODE_DISABLE;
	    if (HAL_DMA_Init(&hdma_adc1) != HAL_OK)
	    {
	      Error_Handler();
	    }

	    __HAL_LINKDMA(hadc,DMA_Handle,hdma_adc1);

	    /* DMA2_Stream4_IRQn interrupt configuration */
	    HAL_NVIC_SetPriority(DMA2_Stream4_IRQn, 5, 0);
	    HAL_NVIC_EnableIRQ(DMA2_Stream4_IRQn);
	    /* USER CODE END ADC1_MspInit 1 */
	}
	else if(hadc->Instance==ADC3)
	{
		/* USER CODE BEGIN ADC3_MspInit 0 */
	    PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_ADC;
	    PeriphClkInitStruct.PLL2.PLL2M = 5;
	    PeriphClkInitStruct.PLL2.PLL2N = 192;
	    PeriphClkInitStruct.PLL2.PLL2P = 40;
	    PeriphClkInitStruct.PLL2.PLL2Q = 2;
	    PeriphClkInitStruct.PLL2.PLL2R = 2;
	    PeriphClkInitStruct.PLL2.PLL2RGE = RCC_PLL2VCIRANGE_2;
	    PeriphClkInitStruct.PLL2.PLL2VCOSEL = RCC_PLL2VCOWIDE;
	    PeriphClkInitStruct.PLL2.PLL2FRACN = 0;
	    PeriphClkInitStruct.AdcClockSelection = RCC_ADCCLKSOURCE_PLL2;
	    if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
	    {
	      Error_Handler();
	    }

		/* USER CODE END ADC3_MspInit 0 */
		/* Peripheral clock enable */
		__HAL_RCC_ADC3_CLK_ENABLE();

		__HAL_RCC_GPIOF_CLK_ENABLE();
		__HAL_RCC_GPIOC_CLK_ENABLE();
		/**ADC3 GPIO Configuration
		 * PF8     ------> ADC3_IN6
		 * PF9     ------> ADC3_IN7
		 * PC2_C     ------> ADC3_INP0
    	 * PC3_C     ------> ADC3_INP1
		 * */
		if(ci_conf[0].type != INPUT_FREQUENCY)//AI 1 adc3 tim 13 port F pin 8 and adc1 port c pin 0
		{
			GPIO_InitStruct.Pin = GPIO_PIN_8;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
		}
		if(ci_conf[1].type != INPUT_FREQUENCY)//AI 2 adc3 tim 14 port F pin 9 and adc1 port c pin 1
		{
			GPIO_InitStruct.Pin = GPIO_PIN_9;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
		}
		/*
		if(ci_conf[4].type != INPUT_FREQUENCY)
		{
			HAL_SYSCFG_AnalogSwitchConfig(SYSCFG_SWITCH_PC2, SYSCFG_SWITCH_PC2_OPEN);
		}
		*/
		HAL_SYSCFG_AnalogSwitchConfig(SYSCFG_SWITCH_PC2, SYSCFG_SWITCH_PC2_OPEN);
	    HAL_SYSCFG_AnalogSwitchConfig(SYSCFG_SWITCH_PC3, SYSCFG_SWITCH_PC3_OPEN);

		/* USER CODE BEGIN ADC3_MspInit 1 */

	    HAL_DMA_DeInit(&hdma_adc3);

	    /* ADC3 DMA Init */
	    /* ADC3 Init */
	    hdma_adc3.Instance = DMA2_Stream1;
	    hdma_adc3.Init.Request = DMA_REQUEST_ADC3;
	    hdma_adc3.Init.Direction = DMA_PERIPH_TO_MEMORY;
	    hdma_adc3.Init.PeriphInc = DMA_PINC_DISABLE;
	    hdma_adc3.Init.MemInc = DMA_MINC_ENABLE;
	    hdma_adc3.Init.PeriphDataAlignment = DMA_PDATAALIGN_WORD;
	    hdma_adc3.Init.MemDataAlignment = DMA_MDATAALIGN_WORD;
	    hdma_adc3.Init.Mode = DMA_CIRCULAR;
	    hdma_adc3.Init.Priority = DMA_PRIORITY_VERY_HIGH;
	    hdma_adc3.Init.FIFOMode = DMA_FIFOMODE_DISABLE;

	    if (HAL_DMA_Init(&hdma_adc3) != HAL_OK)
	    {
	      Error_Handler();
	    }

	    __HAL_LINKDMA(hadc,DMA_Handle,hdma_adc3);

	    /* DMA2_Stream1_IRQn interrupt configuration */
	    HAL_NVIC_SetPriority(DMA2_Stream1_IRQn, 0, 0);
	    HAL_NVIC_EnableIRQ(DMA2_Stream1_IRQn);
		/* USER CODE END ADC3_MspInit 1 */
	}

}
#endif	//SDK_SERVICE_CFG_INPUT
/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
