	/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : DigitalIO_platformservice.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 28/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the DIO platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             28/01/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"

/* USER CODE END Includes */

/* USER CODE BEGIN Macros */

/* USER CODE END Macros */
#if (SDK_SERVICE_DIGITAL_OUTPUT == PS_ENABLE)

/* USER CODE BEGIN Global Variables */
extern pvt_variable pv;
/* USER CODE END Global Variables */


#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief Digital IO deinit
 *
 *  This function will make the DIO pins to low.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void DIO_deinit(void)
{
	/* Make the DIO to OPEN*/
	Set_OutputOpen(1);
	Set_OutputOpen(2);
	Set_OutputOpen(3);
	Set_OutputOpen(4);
}

/** @brief Digital IO reinit
 *
 *  This function will make the DIO to reinit state.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void DIO_reinit(void)
{
	uint8_t out_cfg = 0;
	uint8_t out_state = 0;

	/* Get the previous state */
	Get_DL(DIGITAL_OUTPUT_01_CFG, &out_cfg);
	Get_DL(DIGITAL_OUTPUT_01_STATE, &out_state);

	if ((CONF_LOW_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputLow(1);
	}
	else
	if ((CONF_HIGH_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputHigh(1);
	}

	Get_DL(DIGITAL_OUTPUT_02_CFG, &out_cfg);
	Get_DL(DIGITAL_OUTPUT_02_STATE, &out_state);

	if ((CONF_LOW_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputLow(2);
	}
	else
	if ((CONF_HIGH_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputHigh(2);
	}

	Get_DL(DIGITAL_OUTPUT_03_CFG, &out_cfg);
	Get_DL(DIGITAL_OUTPUT_03_STATE, &out_state);

	if ((CONF_LOW_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputLow(3);
	}
	else
	if ((CONF_HIGH_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputHigh(3);
	}

	Get_DL(DIGITAL_OUTPUT_04_CFG, &out_cfg);
	Get_DL(DIGITAL_OUTPUT_04_STATE, &out_state);

	if ((CONF_LOW_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputLow(4);
	}
	else
	if ((CONF_HIGH_SIDE_DRIVER == out_cfg) && (CONF_DIGITAL_OUTPUT_ON == out_state))
	{
		/* Set the Low Side in the TCA9535 */
		Set_OutputHigh(4);
	}
}
#endif

/** @brief DIO DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return None
 *
 */
int8_t DIO_DB_Init(void)
{
    uint8_t DIO_config;
    /* Set the OutPut State */
    uint8_t DIO_state = CONF_DIGITAL_OUTPUT_OFF;


    DIO_config = CF_DIGITAL_OUTPUT_01_CFG;
    if (!((CONF_LOW_SIDE_DRIVER == DIO_config) || (CONF_HIGH_SIDE_DRIVER == DIO_config)
            || (CONF_OPEN_DRIVE_DRIVER == DIO_config)))
    {
        DIO_config = CONF_OPEN_DRIVE_DRIVER;
    }
    /* Set the Digital Output State*/
    Set_DL(DIGITAL_OUTPUT_01_CFG, &DIO_config);
    pv.doscfg[0] =  DIO_config;

    DIO_config = CF_DIGITAL_OUTPUT_02_CFG;
    if (!((CONF_LOW_SIDE_DRIVER == DIO_config) || (CONF_HIGH_SIDE_DRIVER == DIO_config)
            || (CONF_OPEN_DRIVE_DRIVER == DIO_config)))
    {
        DIO_config = CONF_OPEN_DRIVE_DRIVER;
    }
    /* Set the Digital Output State*/
    Set_DL(DIGITAL_OUTPUT_02_CFG, &DIO_config);
    pv.doscfg[1] =  DIO_config;


    DIO_config = CF_DIGITAL_OUTPUT_03_CFG;
    if (!((CONF_LOW_SIDE_DRIVER == DIO_config) || (CONF_HIGH_SIDE_DRIVER == DIO_config)
            || (CONF_OPEN_DRIVE_DRIVER == DIO_config)))
    {
        DIO_config = CONF_OPEN_DRIVE_DRIVER;
    }
    /* Set the Digital Output State*/
    Set_DL(DIGITAL_OUTPUT_03_CFG, &DIO_config);
    pv.doscfg[2] =  DIO_config;

    DIO_config = CF_DIGITAL_OUTPUT_04_CFG;
    if (!((CONF_LOW_SIDE_DRIVER == DIO_config) || (CONF_HIGH_SIDE_DRIVER == DIO_config)
            || (CONF_OPEN_DRIVE_DRIVER == DIO_config)))
    {
        DIO_config = CONF_OPEN_DRIVE_DRIVER;
    }
    /* Set the Digital Output State*/
    Set_DL(DIGITAL_OUTPUT_04_CFG, &DIO_config);
    pv.doscfg[3] =  DIO_config;

	/* Set the open drive Digital Output */
	Set_DL(DIGITAL_OUTPUT_01_STATE, &DIO_state);
	Set_DL(DIGITAL_OUTPUT_02_STATE, &DIO_state);
	Set_DL(DIGITAL_OUTPUT_03_STATE, &DIO_state);
	Set_DL(DIGITAL_OUTPUT_04_STATE, &DIO_state);

   	/* Set the Open Driver in the TCA9535 */
   	Set_OutputOpen(1);
   	Set_OutputOpen(2);
   	Set_OutputOpen(3);
   	Set_OutputOpen(4);

	return PS_SUCCESS;
}

/** @brief DIO Platform service task
 *
 *  This function starts the DIO Platform Service
 *  task.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
static void DIO_ServiceStart(void * pvParameters)
{
	uint8_t *dobuf;
	uint8_t res;
	uint32_t cNotifiedValue;
	uint8_t func;
	TickType_t  th_timeout = PS_DIO_TASK_TIMEOUT;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(DIO_deinit,NULL,DIO_reinit);
#endif

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)

		/* Ping the watchdog timer */
		res = SetWD(WD_PING_DIO_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("DIO Task failed to ping the WD Timer\r\n");
		}
#endif

		/* Wait for receiving the Event from the SDK API and DIO timer */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, th_timeout);

		if(res != pdTRUE)
		{
			continue;
		}

		if (cNotifiedValue & (1<<DIO_MSG_EVENT))
		{
			/* Receive Message from the DIO SDK API */
			res = xQueueReceive(pv.dotxq, &dobuf , ( TickType_t ) Q_TICKSTOWAIT );
			if(res != pdTRUE)
			{
				break;
			}

			func = dobuf[DIO_FUNC_INDEX];

			/* Update the result */
			dobuf[DIO_RESULT_INDEX] = SUCCESS;

			switch(func)
			{
				case DO_GET:
					/* Update the status */
					dobuf[DIO_STATE_INDEX] = pv.dostate[dobuf[DIO_INDEX_NUM] - 1];
					break;

				case DO_SET:

					switch(dobuf[DIO_SUB_FUNC_INDEX])
					{
						case SET_DIO_CFG:
							/* Set the LOW Side driver */
							if (CONF_LOW_SIDE_DRIVER == dobuf[DIO_STATE_INDEX])
							{
								Set_OutputLow(dobuf[DIO_INDEX_NUM]);

								/* Update the status */
								pv.doscfg[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
							}
							else
							/* Set the High Side driver */
							if (CONF_HIGH_SIDE_DRIVER == dobuf[DIO_STATE_INDEX])
							{
								Set_OutputHigh(dobuf[DIO_INDEX_NUM]);

								/* Update the status */
								pv.doscfg[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
							}
							else
							/* Set the Open drive */
							if (CONF_OPEN_DRIVE_DRIVER == dobuf[DIO_STATE_INDEX])
							{
								Set_OutputOpen(dobuf[DIO_INDEX_NUM]);

								/* Update the status */
								pv.doscfg[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
							}
							else
								dobuf[DIO_RESULT_INDEX] = FAILURE;
							break;

						case SET_DIO_STATE:

							/* Set the DIO state ON */
							if (CONF_DIGITAL_OUTPUT_ON == dobuf[DIO_STATE_INDEX])
							{
								/* Set the LOW Side driver */
								if (CONF_LOW_SIDE_DRIVER == pv.doscfg[dobuf[DIO_INDEX_NUM] - 1])
								{
									Set_OutputLow(dobuf[DIO_INDEX_NUM]);

									/* Update the status */
									pv.dostate[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
								}
								/* Set the High Side driver */
								else
								if (CONF_HIGH_SIDE_DRIVER == pv.doscfg[dobuf[DIO_INDEX_NUM] - 1])
								{
									Set_OutputHigh(dobuf[DIO_INDEX_NUM]);

									/* Update the status */
									pv.dostate[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
								}
								/* Set the Open drive */
								else
								{
									Set_OutputOpen(dobuf[DIO_INDEX_NUM]);

									/* Update the status */
									pv.dostate[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
								}
							}
							else
							/* Set the DIO state OFF */
							if(CONF_DIGITAL_OUTPUT_OFF == dobuf[DIO_STATE_INDEX])
							{
								Set_OutputOpen(dobuf[DIO_INDEX_NUM]);

								/* Update the status */
								pv.dostate[dobuf[DIO_INDEX_NUM] - 1] = dobuf[DIO_STATE_INDEX];
							}
							break;
						}
					break;

				default:
					/* Update the result */
					dobuf[DIO_RESULT_INDEX] = FAILURE;
					break;
			}
			/* Send the response to the SDK API */
			xQueueSend( pv.dorxq, ( void * )&dobuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}


/** @brief Start DIO Platform service
 *
 *  This function initializes the following modules
 *     ===> DIO DB
 *     ===> DIO platform driver
 *     ===> DIO software resources
 *     ===> Start the DIO service
 *  the DIO platform service layer.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t DIO_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create DIO TxQ for receiving value from the SDK API */
	pv.dotxq = xQueueCreate( DIO_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.dotxq)
	{
		LOGERROR("Unable to initialize the Digital Output TxQ\r\n");
		return PS_FAILURE;
	}

	/* Create DIO RxQ for sending return value to the SDK API */
	pv.dorxq = xQueueCreate( DIO_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.dorxq)
	{
		LOGERROR("Unable to initialize the Digital Output RxQ\r\n");
		return PS_FAILURE;
	}

	xReturned = xTaskCreate(DIO_ServiceStart, "Digital_Output",
			512, NULL, PS_DIO_TASK_PRIORITY,   &pv.do_task);

	/* Start the DIO Platform service thread */
	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the Digital Output Platform service thread\r\n");
		return PS_FAILURE;
	}

	LOGERROR("Digital Output Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
