/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : DL_u16_Validation.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Data Layer unsigned 16 bits field validation function

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../Maximatecc/Inc/Data_Layer.h"
#include "../Inc/Common.h"
#include "../Inc/Dl_U16_Validation.h"
/* USER CODE END Includes */


/* USER CODE BEGIN PV */
int8_t (*dl_u16_func  [(END_U16BIT   -1)    - START_U16BIT  ])( uint16_t, uint16_t);

extern pvt_variable pv;
/* USER CODE END PV */

/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/** @brief DB Validation function
 *
 *  This function initialize the DB Validation function for the each
 *  unsinged 16 bits field.
 *
 *  @param  None
 *
 *  @return None
 *
 */
void DB_u16_Field_Validation_Reg(void)
{
	/* Clear the Callback memory */
	memset (&dl_u16_func[0], 0x00, ((END_U16BIT   -1)    - START_U16BIT));

#if (SDK_SERVICE_LCD == PS_ENABLE)

	/* DISPLAY_X0_ORIGIN Validation for the Alpha Layer */
	dl_u16_func[DISPLAY_X0_ORIGIN - (START_U16BIT + 1)] = validate_display_x0_origin_val;

	/* DISPLAY_Y0_ORIGIN Validation for the Alpha Layer */
	dl_u16_func[DISPLAY_Y0_ORIGIN - (START_U16BIT + 1)] = validate_display_y0_origin_val;

#else

	/* DISPLAY_X0_ORIGIN_DUMMY Validation for the Alpha Layer */
	dl_u16_func[DISPLAY_X0_ORIGIN_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* DISPLAY_Y0_ORIGIN_DUMMY Validation for the Alpha Layer */
	dl_u16_func[DISPLAY_Y0_ORIGIN_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

#endif

#if (SDK_SERVICE_LED == PS_ENABLE)

#if (CONF_RED_LED_PS_STATE == PS_ENABLE)
	/* LED_RED_BLINKING Validation */
	dl_u16_func[LED_RED_BLINKING - (START_U16BIT + 1)] = validate_led_blinking;
#else
	/* LED_RED_BLINKING_DUMMY Validation */
	dl_u16_func[LED_RED_BLINKING_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;
#endif

#if (CONF_AMB_LED_PS_STATE == PS_ENABLE)
	/* LED_AMB_BLINKING Validation */
	dl_u16_func[LED_AMB_BLINKING - (START_U16BIT + 1)] = validate_led_blinking;
#else
	/* LED_AMB_BLINKING_DUMMY Validation */
	dl_u16_func[LED_AMB_BLINKING_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;
#endif

#endif	//SDK_SERVICE_LED

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

	/* CFG_INPUT_01_VOLTAGE Validation */

	/* CFG_INPUT_01_RESISTANCE Validation */
	dl_u16_func[CFG_INPUT_01_RESISTANCE - (START_U16BIT + 1)] = validate_cfg_resistance;

	/* CFG_INPUT_02_RESISTANCE Validation */
	dl_u16_func[CFG_INPUT_02_RESISTANCE - (START_U16BIT + 1)] = validate_cfg_resistance;

	/* CFG_INPUT_03_RESISTANCE Validation */
	dl_u16_func[CFG_INPUT_03_RESISTANCE - (START_U16BIT + 1)] = validate_cfg_resistance;

	/* CFG_INPUT_04_RESISTANCE Validation */
	dl_u16_func[CFG_INPUT_04_RESISTANCE - (START_U16BIT + 1)] = validate_cfg_resistance;

	/* CFG_INPUT_05_RESISTANCE Validation */
	dl_u16_func[CFG_INPUT_05_RESISTANCE - (START_U16BIT + 1)] = validate_cfg_resistance;

	/* CFG_INPUT_01_VOLTAGE_32V Validation */
	dl_u16_func[CFG_INPUT_01_VOLTAGE_32V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_01_VOLTAGE_LOW_6V Validation */
	dl_u16_func[CFG_INPUT_01_VOLTAGE_LOW_6V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_02_VOLTAGE_32V Validation */
	dl_u16_func[CFG_INPUT_02_VOLTAGE_32V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_02_VOLTAGE_LOW_6V Validation */
	dl_u16_func[CFG_INPUT_02_VOLTAGE_LOW_6V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_03_VOLTAGE_32V Validation */
	dl_u16_func[CFG_INPUT_03_VOLTAGE_32V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_03_VOLTAGE_LOW_6V Validation */
	dl_u16_func[CFG_INPUT_03_VOLTAGE_LOW_6V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_04_VOLTAGE_32V Validation */
	dl_u16_func[CFG_INPUT_04_VOLTAGE_32V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_04_VOLTAGE_LOW_6V Validation */
	dl_u16_func[CFG_INPUT_04_VOLTAGE_LOW_6V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_05_VOLTAGE_32V Validation */
	dl_u16_func[CFG_INPUT_05_VOLTAGE_32V - (START_U16BIT + 1)] = validate_cfg_voltage;

	/* CFG_INPUT_05_VOLTAGE_LOW_6V Validation */
	dl_u16_func[CFG_INPUT_05_VOLTAGE_LOW_6V - (START_U16BIT + 1)] = validate_cfg_voltage;

#else

	/* CFG_INPUT_01_VOLTAGE_DUMMY Validation */
	//dl_u16_func[CFG_INPUT_01_VOLTAGE_3_3V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_01_RESISTANCE_DUMMY Validation */
	dl_u16_func[CFG_INPUT_01_RESISTANCE_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_02_VOLTAGE_DUMMY Validation */
	//dl_u16_func[CFG_INPUT_02_VOLTAGE_3_3V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_02_RESISTANCE_DUMMY Validation */
	dl_u16_func[CFG_INPUT_02_RESISTANCE_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_03_VOLTAGE_DUMMY Validation */
	//dl_u16_func[CFG_INPUT_03_VOLTAGE_3_3V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_03_RESISTANCE_DUMMY Validation */
	dl_u16_func[CFG_INPUT_03_RESISTANCE_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_04_VOLTAGE_DUMMY Validation */
	//dl_u16_func[CFG_INPUT_04_VOLTAGE_3_3V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_04_RESISTANCE_DUMMY Validation */
	dl_u16_func[CFG_INPUT_04_RESISTANCE_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_05_VOLTAGE_DUMMY Validation */
	//dl_u16_func[CFG_INPUT_05_VOLTAGE_3_3V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_05_RESISTANCE_DUMMY Validation */
	dl_u16_func[CFG_INPUT_05_RESISTANCE_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_01_VOLTAGE_32V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_01_VOLTAGE_32V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_01_VOLTAGE_LOW_6V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_01_VOLTAGE_LOW_6V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_02_VOLTAGE_32V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_02_VOLTAGE_32V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_02_VOLTAGE_LOW_6V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_02_VOLTAGE_LOW_6V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_03_VOLTAGE_32V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_03_VOLTAGE_32V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_03_VOLTAGE_LOW_6V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_03_VOLTAGE_LOW_6V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_04_VOLTAGE_32V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_04_VOLTAGE_32V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_04_VOLTAGE_LOW_6V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_04_VOLTAGE_LOW_6V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_05_VOLTAGE_32V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_05_VOLTAGE_32V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* CFG_INPUT_05_VOLTAGE_LOW_6V_DUMMY Validation */
	dl_u16_func[CFG_INPUT_05_VOLTAGE_LOW_6V_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

#endif	//SDK_SERVICE_CFG_INPUT

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)

	/* BATTERY_LEVEL Validation */
	dl_u16_func[BATTERY_LEVEL - (START_U16BIT + 1)] = validate_battery_level;

#else

	/* BATTERY_LEVEL_DUMMY Validation */
	dl_u16_func[BATTERY_LEVEL_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

#endif	//SDK_SERVICE_POWER_MONITOR

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)

	/* TIMER_TIMEOUT_01 Validation */
	dl_u16_func[TIMER_TIMEOUT_01 - (START_U16BIT + 1)] = validate_timer_timeout;

	/* TIMER_TIMEOUT_02 Validation */
	dl_u16_func[TIMER_TIMEOUT_02 - (START_U16BIT + 1)] = validate_timer_timeout;

	/* TIMER_TIMEOUT_03 Validation */
	dl_u16_func[TIMER_TIMEOUT_03 - (START_U16BIT + 1)] = validate_timer_timeout;

	/* TIMER_TIMEOUT_04 Validation */
	dl_u16_func[TIMER_TIMEOUT_04 - (START_U16BIT + 1)] = validate_timer_timeout;

	/* TIMER_TIMEOUT_05 Validation */
	dl_u16_func[TIMER_TIMEOUT_05 - (START_U16BIT + 1)] = validate_timer_timeout;

	/* TIMER_TIMEOUT_06 Validation */
	dl_u16_func[TIMER_TIMEOUT_06 - (START_U16BIT + 1)] = validate_timer_timeout;

#else

	/* TIMER_TIMEOUT_01_DUMMY Validation */
	dl_u16_func[TIMER_TIMEOUT_01_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* TIMER_TIMEOUT_02_DUMMY Validation */
	dl_u16_func[TIMER_TIMEOUT_02_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* TIMER_TIMEOUT_03_DUMMY Validation */
	dl_u16_func[TIMER_TIMEOUT_03_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* TIMER_TIMEOUT_04_DUMMY Validation */
	dl_u16_func[TIMER_TIMEOUT_04_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* TIMER_TIMEOUT_05_DUMMY Validation */
	dl_u16_func[TIMER_TIMEOUT_05_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

	/* TIMER_TIMEOUT_06_DUMMY Validation */
	dl_u16_func[TIMER_TIMEOUT_06_DUMMY - (START_U16BIT + 1)] = validate_u16_dummy_field;

#endif	//SDK_SERVICE_SWTIMER

}

#if (SDK_SERVICE_LED == PS_ENABLE)

/** @brief validation led blinking
 *
 *  This function validates the led blinking and
 *  sends the led blinking to platform service.
 *
 *  @param  led_idx [IN]      : LED DB Field
 *          blink_period [IN] : led blink period
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_led_blinking(uint16_t led_idx, uint16_t blink_period)
{
	/* Check LED blink value */
	if(!((LED_MIN_BLK_TIME <= blink_period) && (LED_MAX_BLK_TIME > blink_period)))
		return FAILURE;

	return SUCCESS;
}

#endif	//SDK_SERVICE_LED

#if (SDK_SERVICE_CFG_INPUT == PS_ENABLE)

/** @brief validate the voltage
 *
 *  This function validates the voltage value and
 *  return the status.
 *
 *  @param  cfg_idx [IN]   : configurable DB Field
 *          voltage [IN]   : voltage value
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_cfg_voltage(uint16_t cfg_idx, uint16_t voltage)
{
	if(!((MIN_VOLTAGE_RANGE <= voltage) && (MAX_VOLTAGE_RANGE > voltage)))
		return FAILURE;

	return SUCCESS;
}

/** @brief validate the resistance
 *
 *  This function validates the resistance value and
 *  return the status.
 *
 *  @param  cfg_idx [IN]    : configurable DB Field.
 *          resistance [IN] : resistance value.
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_cfg_resistance(uint16_t cfg_idx, uint16_t resistance)
{
	if(!((MIN_RESISTANCE_RANGE <= resistance) &&
			(MAX_RESISTANCE_RANGE > resistance)))
		return FAILURE;

	return SUCCESS;
}

#endif	//SDK_SERVICE_CFG_INPUT

#if (SDK_SERVICE_POWER_MONITOR == PS_ENABLE)

/** @brief validation of Battery level
 *
 *  This function validates the Battery level and
 *  sends the Battery level value to platform service.
 *
 *  @param  battery_idx [IN]   : Battery DB Field
 *          battery_level [IN] : Battery Level
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_battery_level(uint16_t battery_idx, uint16_t battery_level)
{
	/* Battery level value */
	if (!((BATTERY_MIN_VAL <= battery_level) &&
			(BATTERY_MAX_VAL >= battery_level)))
		return FAILURE;

	return SUCCESS;
}

#endif	//SDK_SERVICE_POWER_MONITOR

#if (SDK_SERVICE_SWTIMER == PS_ENABLE)

/** @brief validation of Timer Timeout value
 *
 *  This function validates the timer timeout value and
 *  sends the timer timeout value to platform service.
 *
 *  @param  db_idx      [IN] : Timer DB index
 *          timeout_val [IN] : Timeout value
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_timer_timeout(uint16_t db_idx, uint16_t timeout_val)
{
	if (!((SWT_MAX_MS >= timeout_val) && (SWT_MIN_MS <= timeout_val)))
		return FAILURE;

	return SUCCESS;
}
#endif	//SDK_SERVICE_SWTIMER

#if (SDK_SERVICE_LCD == PS_ENABLE)
/** @brief validate the display x0 origin value
 *
 *  This function validates the display x0 origin value.
 *
 *  @param  db_idx      	[IN] : Camera DB index
 *          disp_origin_x0	[IN] : display origin value
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_display_x0_origin_val(uint16_t db_idx, uint16_t disp_origin_x0)
{
	/* Validate the LCD X0 Origin */
	if (LCD_RESOlUTION_X0 < disp_origin_x0)
		return FAILURE;

	return SUCCESS;
}

/** @brief validate the display y0 origin value
 *
 *  This function validates the display y0 origin value.
 *
 *  @param  db_idx      	[IN] : Camera DB index
 *          disp_origin_y0	[IN] : display origin value
 *
 *  @return  1 : SUCCESS
 *           0 : FAILURE
 *
 */
int8_t validate_display_y0_origin_val(uint16_t db_idx, uint16_t disp_origin_y0)
{
	/* Validate the LCD Y0 Origin */
	if (LCD_RESOlUTION_Y0 < disp_origin_y0)
		return FAILURE;

	return SUCCESS;
}

#endif	//SDK_SERVICE_CAMERA

/** @brief u16 Dummy Field validation
 *
 *  This function validates the dummy field in the DB.
 *
 *  @param  dummy1 [IN] : Not used
 *          dummy2 [IN] : Not used
 *
 *  @return 0 : FAILURE
 *
 */
int8_t validate_u16_dummy_field(uint16_t dummy1, uint16_t dummy2)
{
	return FAILURE;
}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
