/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Led_Platformservice.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LED platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/Common.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Global Variables */
#if (SDK_SERVICE_LED == PS_ENABLE)
extern pvt_variable pv;
LED_cfg_info ledcfg[MAX_LED_NUM] =
{
		/*Enable, LED State, Duration to blinking, Current timer count, blinking state */
		{CONF_RED_LED_PS_STATE, CONF_RED_LED_STATE, CONF_RED_LED_BLINKING_MS, 0, 0},
		{CONF_AMB_LED_PS_STATE, CONF_AMB_LED_STATE, CONF_AMB_LED_BLINKING_MS, 0, 0}
};


#if (SDK_SERVICE_PM == PS_ENABLE)
/** @brief LED module deinit
 *
 *  This function will make the LED Enable pin to low.
 *  (The chip enters hardware shutdown mode
 *   when the LED enabled pin is pulled low)
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void led_deinit(void)
{
	/* If LEDs are ON, OFF the LED */
    uint8_t index;
    for (index = 0; index < MAX_LED_NUM; index++)
    {
    	if(LED_ON & ledcfg[index].led_state)
    	{
    		/* Turn off the LED */
    		Led_Off(index+1);
    	}
    }
}

/** @brief LED reinit
 *
 *  This function will make the LED enable pin to high.
 *  (The chip enters normal mode when the
 *  LED enable pin is pulled high)
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void led_reinit(void)
{
	/* If LEDs are ON, OFF the LED */
    uint8_t index;
    for (index = 0; index < MAX_LED_NUM; index++)
    {
    	if(LED_ON & ledcfg[index].led_state)
    	{
    		/* Turn off the LED */
    		Led_On(index+1);
    	}
    }
}
#endif	//SDK_SERVICE_PM

/** @brief LED Timer Callback
 *
 *  This function is used for the LED blink
 *  functionality
 *
 *  @param  xTimer : LED timer handle
 *
 *  @return none
 *
 */
void LEDTimerCallback( TimerHandle_t xTimer )
{
	/* Notify to the LED task to blink */
	xTaskNotify( pv.led_task, (1<<LED_TMR_EVENT), eSetBits );
}

/** @brief led Blinking
 *
 *  Blink the led with timer callback
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
static int8_t led_Blinking(void)
{
    uint8_t index;
    uint8_t enble_blnk_tmr = 0;

    for (index = 0; index < MAX_LED_NUM; index++)
    {
    	if(LED_OFF & ledcfg[index].led_state)
            continue;

        /* LED Blinking is Zero */
        if (0 == ledcfg[index].blk_period)
        	continue;

        ledcfg[index].timer_count++;

        /* Check for blinking timer elapse counter */
        if (ledcfg[index].timer_count >= ledcfg[index].blk_period )
        {
        	/* Reset the counter */
        	ledcfg[index].timer_count = 0;

        	if (LED_ON == ledcfg[index].blk_state)
        	{
                /* Update the Next LED state */
        		ledcfg[index].blk_state = LED_OFF;

        		/* Turn off the LED */
				Led_Off(index+1);

        	}
        	else
        	{
                /* Update the Next LED state */
        		ledcfg[index].blk_state = LED_ON;

        		/* Turn on the LED */
				Led_On(index+1);

        	}
        }

        /* Enable the blinking timer again */
        enble_blnk_tmr = 1;
    }

    /* Check whether the blinking timer has to be stopped */
    if (0 == enble_blnk_tmr)
    {
		 /* STOP LED Blinking timer */
		 xTimerStop( pv.xTimers[ PS_LED_TIMER ], 0 );
    }
	return PS_SUCCESS;
}



/** @brief LED DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t LED_DB_Init(void)
{
#if(CONF_RED_LED_PS_STATE == PS_ENABLE)
    uint8_t   led_db1 = LED_RED_STATE;
    uint16_t  led_db2 = LED_RED_BLINKING;
#else
    uint8_t   led_db1 = LED_RED_STATE_DUMMY;
    uint16_t  led_db2 = LED_RED_BLINKING_DUMMY;
#endif
    uint8_t   index;

    if (MAX_LED_NUM > MAX_PLATFORM_SUPPORTED_LED)
    	return PS_FAILURE;

    /* Initialize the LED DB Variables */
    for (index = 0; index < MAX_LED_NUM; index++)
    {
    	/* Check the LED PS state */
        if (LED_ENABLE != ledcfg[index].led_enable)
    	{
        	ledcfg[index].led_state = LED_DISABLE;
    	}

        /* Check the LED ON/OFF state */
        if ((LED_ON != ledcfg[index].led_state) &&
        		(LED_OFF != ledcfg[index].led_state))
    	{
        	ledcfg[index].led_state = LED_OFF;
    	}

        /* Check the LED Blinking period */
        if ((0 >= ledcfg[index].blk_period) &&
        		(LED_MAX_BLK_TIME <= ledcfg[index].blk_period))
    	{
        	ledcfg[index].blk_period = 0;
    	}

        /* Update default (xx_LED_BLINKING) DB Field  */
        Set_DL(led_db2++, (uint8_t *)&ledcfg[index].blk_period);

        /* Update default (xx_LED_STATE) DB Field  */
        Set_DL(led_db1++, &ledcfg[index].led_state);

    }

    return PS_SUCCESS;
}

/** @brief LED Platform service task
 *
 *  This function starts the LED Platform Service
 *  task.
 *
 *  @param  pvParameters - Private data pointer
 *
 *  @return none
 *
 */
void LED_ServiceStart(void *pvParameters)
{
	uint8_t  *ledbuf;
	uint8_t   index;
	uint32_t cNotifiedValue;
	int8_t   res = 0;
	uint8_t  func;
	TickType_t  th_timeout = PS_LED_TASK_TIMEOUT;

	/* Initialize the LED based on the Initial configuration Value */
    for (index = 0; index < MAX_LED_NUM; index++)
    {
		/* Check the blinking time value is present  */
		if((ledcfg[index].blk_period == 0) &&
				(ledcfg[index].led_state & LED_ON))
		{
			/* Turn on the LED */
			Led_On(index+1);
		}
		else
		/* LED Blinking period is non zero and LED is ON */
		if((0 != ledcfg[index].blk_period) &&
				(LED_ON & ledcfg[index].led_state))
		{
			/* Turn on the LED */
			Led_On(index + 1);

			/* Update the NEXT state */
			ledcfg[index].blk_state = LED_ON;

        	/* Reset the counter */
			ledcfg[index].timer_count = 0;

			/* Start LED Blinking timer */
			xTimerStart( pv.xTimers[ PS_LED_TIMER ], 0 );
		}
    }

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_LED_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("LED Task failed to ping the WD Timer\r\n");
		}
#endif
		/* Wait for receiving the Event from the SDK API and LED timer */
		res = xTaskNotifyWait( CLEAR_BITS_ONENTRY, ULONG_MAX , &cNotifiedValue, th_timeout);

		if(res != pdTRUE)
		{
			continue;
		}

		/* Event from SDK LED API */
		if (cNotifiedValue & (1<<LED_TMR_EVENT))
		{
			/* Check for the LED Blinking */
			led_Blinking();
		}

		if (cNotifiedValue & (1<<LED_MSG_EVENT))
		{
			/* Receive Message from the LED SDK API */
			res = xQueueReceive(pv.ledtxq, &ledbuf , ( TickType_t ) Q_TICKSTOWAIT );

			if(res != pdTRUE)
			{
				break;
			}

			func = ledbuf[LED_FUNC_INDEX];

			/* Update the result */
			ledbuf[LED_RESULT_INDEX] = SUCCESS;

			switch(func)
			{
				case LED_ACT:
					/* Update the LED blinking period */
					memcpy(&ledcfg[ledbuf[LED_NUM_INDEX+1]].blk_period,
							&ledbuf[LED_BLK_INDEX], sizeof(uint16_t));
					break;

				case LED_SET:

					if (LED_ON != ledbuf[LED_STATE_INDEX+1])
					{
						/* Update the LED state */
						ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state &= ~LED_ON;
						ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state |= LED_OFF;
					}
					else
					{
						ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state &= ~LED_OFF;
						ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state |= LED_ON;
					}

					/* Update the Blinking time */
					memcpy(&ledcfg[ledbuf[LED_NUM_INDEX+1]].blk_period,
							&ledbuf[LED_BLK_INDEX+1], sizeof(uint16_t));

					/* Check the blinking time value is present  */
					if((ledcfg[ledbuf[LED_NUM_INDEX+1]].blk_period == 0) &&
							(ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state & LED_ON))
					{
						/* Turn on the LED */
						Led_On(ledbuf[LED_NUM_INDEX+1]+1);

					}
					else
					{
						/* LED Blinking period is non zero and LED is ON */
						if((0 != ledcfg[ledbuf[LED_NUM_INDEX+1]].blk_period) &&
								(LED_ON & ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state))
						{
							/* Turn on the LED */
							Led_On(ledbuf[LED_NUM_INDEX+1] + 1);

							/* Update the NEXT state */
							ledcfg[ledbuf[LED_NUM_INDEX+1]].blk_state = LED_ON;

				        	/* Reset the counter */
							ledcfg[ledbuf[LED_NUM_INDEX+1]].timer_count = 0;

				        	if (pdFALSE == xTimerIsTimerActive(pv.xTimers[PS_LED_TIMER]))
							{
								/* Start LED Blinking timer */
								xTimerStart( pv.xTimers[ PS_LED_TIMER ], 0 );
							}
						}
						else
							/* Turn OFF LED */
							if(LED_OFF & ledcfg[ledbuf[LED_NUM_INDEX+1]].led_state)
							{
								/* Turn off the LED */
								Led_Off(ledbuf[LED_NUM_INDEX+1]+1);
							}
					}
					break;

				default:
					/* Update the result */
					ledbuf[LED_RESULT_INDEX] = FAILURE;
					break;
			}
			/* Send the response to the SDK API */
			xQueueSend( pv.ledrxq, ( void * )&ledbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}

		/* Clear the Notify Value */
		cNotifiedValue = 0;
	}
}

/** @brief Start LED Platform service
 *
 *  This function initializes the following modules
 *     ===> LED DB
 *     ===> LED platform driver
 *     ===> LED software resources
 *     ===> Start the LED service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *         -1 : PS_FAILURE
 *
 */
int8_t LED_PS_Start(void)
{
	BaseType_t xReturned;

#if (SDK_SERVICE_PM == PS_ENABLE)
	/* Register the Power Mode callback */
	PM_Register_Callback(led_deinit,NULL,led_reinit);
#endif

	/* Create the LED timer for blinking */
	pv.xTimers[ PS_LED_TIMER ] = xTimerCreate
	                   ( "LED_Timer",
	                	PS_LED_TIMER_COUNT,
	                    pdTRUE,
	                    ( void * ) 0,
	                    LEDTimerCallback
	                   );

	if (NULL == pv.xTimers[ PS_LED_TIMER ])
	{
		LOGERROR("Unable to initialize the LED timer\r\n");
		return PS_FAILURE;
	}

	/* Create LED TxQ for receiving value from the SDK API */
	pv.ledtxq = xQueueCreate( LED_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.ledtxq)
	{
		LOGERROR("Unable to initialize the LED TxQ \r\n");
		return PS_FAILURE;
	}

	/* Create LED RxQ for sending return value to the SDK API */
	pv.ledrxq = xQueueCreate( LED_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.ledrxq)
	{
		LOGERROR("Unable to initialize the LED RxQ \r\n");
		return PS_FAILURE;
	}

	/* Create the LED task */
	xReturned = xTaskCreate(LED_ServiceStart, "LED_Service",
			LED_TASK_SIZE, NULL, PS_LED_TASK_PRIORITY,   &pv.led_task);

    if( pdPASS != xReturned )
    {
		LOGERROR("Unable to initialize the LED Task\r\n");
		return PS_FAILURE;
    }

    LOGERROR("LED Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}

#endif	//SDK_SERVICE_LED

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
