/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : PM_platformservice.c
   Author             : Arjun
   Version            : V0.0.0.1
   Origination Date   : 11/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Power Management platform service initialization

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             11/02/2022			Arjun

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../Inc/common.h"
#include "../../../Drivers/Maximatecc/inc/LCD.h"
/* USER CODE END Includes */

/* USER CODE BEGIN Macros */
extern QSPI_HandleTypeDef hqspi;
extern SPI_HandleTypeDef hspi1;
extern ADC_HandleTypeDef hadc1;
extern ADC_HandleTypeDef hadc3;
extern I2C_HandleTypeDef hi2c3;
extern TIM_HandleTypeDef htim1;
extern CRC_HandleTypeDef hcrc;
extern RTC_HandleTypeDef hrtc;
extern FDCAN_HandleTypeDef hfdcan1;

extern SRAM_HandleTypeDef hsram1;
extern SPI_HandleTypeDef hspi2;


/* USER CODE END Macros */


/* USER CODE BEGIN Global Variables */


#if (SDK_SERVICE_PM == PS_ENABLE)
extern pvt_variable pv;


/** @brief Platform Management Callback
 *
 *  This function registers the Platform service
 *  callback functions. These functions will be called
 *  during the Power down mode.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void PM_Register_Callback(void (*DeInit)(void), void(*WakeupSource)(void),
		                      void (*ReInit)(void))
{
	/* Register the callback */
	pv.PM_CB[pv.num_cb].DeInit = DeInit;
	pv.PM_CB[pv.num_cb].WakeupSource = WakeupSource;
	pv.PM_CB[pv.num_cb].ReInit = ReInit;

	pv.num_cb++;
}

static void Set_inputs_high_impedance(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	/*Configure GPIO pin : KEY1_Pin */
	GPIO_InitStruct.Pin = KEY_DATA_PIN_1;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(KEY_DATA_PORT_1, &GPIO_InitStruct);

	/*Configure GPIO pins : KEY2_Pin KEY3_Pin */
	GPIO_InitStruct.Pin = KEY_DATA_PIN_2|KEY_DATA_PIN_3;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

	/*Configure GPIO pin : KEY4_Pin */
	GPIO_InitStruct.Pin = KEY_DATA_PIN_4;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(KEY_DATA_PORT_4, &GPIO_InitStruct);

	/*Configure GPIO pin : BLE_WKP_Pin */
	GPIO_InitStruct.Pin = GPIO_PIN_6;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_1;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_6;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_2;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_2;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOH, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_0;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_7;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOI, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_9;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = GPIO_PIN_10;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);

}

/** @brief Platform Service DeInit
 *
 *  This function calls one by one platform service function.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void PM_DeInit()
{
	uint8_t index;

	/* Call one by one Platform Service DeInit function */
    for (index = 0; index < pv.num_cb; index++)
    {
    	if (NULL != pv.PM_CB[index].DeInit)
    		/* Call Driver Platform Service Deinit function one by one */
    		pv.PM_CB[index].DeInit();
    }

    HAL_MspDeInit();
    ST7789V_DisplayOff();
    HAL_Delay(300);
    HAL_GPIO_WritePin(GPIOE, GPIO_PIN_4, GPIO_PIN_RESET);
    HAL_Delay(300);
    HAL_SRAM_DeInit(&hsram1);
    HAL_SPI_DeInit(&hspi1);
    HAL_SPI_DeInit(&hspi2);
    HAL_QSPI_DeInit(&hqspi);
    HAL_GPIO_WritePin(GPIOG, GPIO_PIN_12, GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_5,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_4,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOD, GPIO_PIN_7,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOF, GPIO_PIN_7,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOI, GPIO_PIN_8,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOG, GPIO_PIN_5,  GPIO_PIN_SET);
    HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOG, GPIO_PIN_14, GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOC, GPIO_PIN_6,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOH, GPIO_PIN_5,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOG, GPIO_PIN_3,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOH, GPIO_PIN_9,  GPIO_PIN_RESET);
    HAL_GPIO_WritePin(GPIOF, GPIO_PIN_0,  GPIO_PIN_RESET);
    HAL_ADC_Stop_DMA(&hadc3);
    HAL_ADC_Stop_DMA(&hadc1);
    Set_inputs_high_impedance();

    __HAL_RCC_GPIOI_CLK_DISABLE();
    __HAL_RCC_GPIOF_CLK_DISABLE();
    __HAL_RCC_GPIOH_CLK_DISABLE();
    __HAL_RCC_GPIOA_CLK_DISABLE();
    __HAL_RCC_GPIOG_CLK_DISABLE();

    HAL_NVIC_DisableIRQ(DMA1_Stream0_IRQn);

    /* DMA1_Stream1_IRQn interrupt configuration */
    HAL_NVIC_DisableIRQ(DMA1_Stream1_IRQn);

    /* DMA1_Stream2_IRQn interrupt configuration */
    HAL_NVIC_DisableIRQ(DMA1_Stream2_IRQn);

    /* DMA1_Stream3_IRQn interrupt configuration */
    HAL_NVIC_DisableIRQ(DMA1_Stream3_IRQn);

    /* DMA2_Stream0_IRQn interrupt configuration */
    HAL_NVIC_DisableIRQ(DMA2_Stream0_IRQn);

    /* DMA2_Stream4_IRQn interrupt configuration */
    HAL_NVIC_DisableIRQ(DMA2_Stream4_IRQn);

     __HAL_RCC_FMC_CLK_DISABLE();

    HAL_PWREx_EnableFlashPowerDown();
}

/** @brief Configure the Wakeup source
 *
 *  This function calls the appropriate Platform service
 *  Layer of the wakeup source function. The driver platform service
 *  layer takes care of configuring the device Wakeup source.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void PM_Conf_Wakeup_Source()
{
	uint8_t index;

    __HAL_RCC_GPIOC_CLK_ENABLE();
    __HAL_RCC_GPIOD_CLK_ENABLE();
    __HAL_RCC_GPIOG_CLK_ENABLE();

	/* Call one by one Platform Service DeInit function */
    for (index = 0; index < pv.num_cb; index++)
    {
    	if (NULL != pv.PM_CB[index].WakeupSource)
    		/* Call Driver Platform Service WakeupSource function one by one */
    		pv.PM_CB[index].WakeupSource();
    }

}

/** @brief ReConfigure the Platform device
 *
 *  This function reconfigures the platform service device
 *  It calls the appropriate reconfigure Platform service
 *  driver function.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void PM_ReInit()
{
	uint8_t index;

    /* Disable all the GPIOs */
/*	__HAL_RCC_GPIOA_CLK_ENABLE();
    __HAL_RCC_GPIOB_CLK_ENABLE();
    __HAL_RCC_GPIOC_CLK_ENABLE();
    __HAL_RCC_GPIOD_CLK_ENABLE();
    __HAL_RCC_GPIOE_CLK_ENABLE();
    __HAL_RCC_GPIOF_CLK_ENABLE();
    __HAL_RCC_GPIOG_CLK_ENABLE();
    __HAL_RCC_GPIOH_CLK_ENABLE();
*/
	/* Call one by one Platform Service ReInit function */
    for (index = 0; index < pv.num_cb; index++)
    {
    	if (NULL != pv.PM_CB[index].ReInit)
    		/* Call Driver Platform Service ReInit function one by one */
    		pv.PM_CB[index].ReInit();
    }

}

/** @brief Power Management Callback Clear
 *
 *  This function clear the callback funcion.
 *
 *  @param  None
 *
 *  @return  None
 *
 */
void PM_CB_Clear(void)
{
	/* Clear the PM callback functions */
	memset(&pv.PM_CB[PM_FUNC_INDEX], 0x00, sizeof(pv.PM_CB));
	pv.num_cb = 0;
}

/** @brief Power Management DB Initialization
 *
 *  This function takes the default configuration from the config header
 *  file and store the data into the DB.
 *
 *  @param  None
 *
 *  @return  0 : PS_SUCCESS
 *          -1 : PS_FAILURE
 *
 */
int8_t PM_DB_Init(void)
{
	uint8_t val = 0;

#if(KEYPAD01_WAKEUP_SOURCE == PS_ENABLE)
	val = TRUE;
	Set_DL(KEYPAD_01_WAKEUP, &val);
#endif

#if(KEYPAD02_WAKEUP_SOURCE == PS_ENABLE)
	val = TRUE;
	Set_DL(KEYPAD_02_WAKEUP, &val);
#endif

#if(KEYPAD03_WAKEUP_SOURCE == PS_ENABLE)
	val = TRUE;
	Set_DL(KEYPAD_03_WAKEUP, &val);
#endif

#if(KEYPAD04_WAKEUP_SOURCE == PS_ENABLE)
	val = TRUE;
	Set_DL(KEYPAD_04_WAKEUP, &val);
#endif

#if(RTC_WAKEUP_SOURCE_STATE == PS_ENABLE)
	uint32_t timeout;

	val = TRUE;
	Set_DL(RTC_WAKEUP, &val);

	timeout = RTC_WAKEUP_SOURCE_TIMEOUT;
	Set_DL(RTC_WAKEUP_TIMEOUT, (uint8_t *)&timeout);
#endif

	return PS_SUCCESS;
}


/** @brief Power Management Platform service task
 *
 *  This function starts the Power Management Platform Service
 *  task.
 *
 *  @param  pvParameters : Task Private structure
 *
 *  @return none
 *
 */
void PM_ServiceStart(void *pvParameters)
{
	uint32_t cNotifiedValue;
	uint8_t *pmbuf;
	uint8_t res;
	uint8_t func;

	while(1)
	{

#if (SDK_SERVICE_WATCHDOG == PS_ENABLE)
		/* Ping the watchdog timer */
		res = SetWD(WD_PING_POWERMODE_TASK);

		if(res != SUCCESS)
		{
			LOGERROR("Power Mode Task failed to ping the WD Timer\r\n");
		}
#endif

		/* Wait for receiving the Event from the SDK API */
		res = xTaskNotifyWait( 0x00, ULONG_MAX , &cNotifiedValue, PS_PM_TASK_TIMEOUT);

		if(res != pdTRUE)
		{
			continue;
		}

		if (cNotifiedValue & (1<<PM_MSG_EVENT))
		{
			/* Receive Message from the Power Mode SDK API */
			res = xQueueReceive(pv.pmtxq, &pmbuf , ( TickType_t ) Q_TICKSTOWAIT );

			if(res != pdTRUE)
			{
				break;
			}

			func = pmbuf[PM_FUNC_INDEX];

			/* Update the result */
			pmbuf[PM_RESULT_INDEX] = SUCCESS;

			switch(func)
			{
				case PM_SET:

					switch(pmbuf[PM_MODE_INDEX])
					{
						case PM_SLEEP_MODE:

							/* Call Sleep Mode Functionality */
							SleepMode();

							/* Update the State in the DB*/
							func = PM_NORMAL_MODE;
							Set_DL(POWER_MODE, &func);

							break;

						case PM_STOP_MODE:

							/* Call Stop Mode Functionality */
							StopMode();

							/* Update the State in the DB*/
							func = PM_NORMAL_MODE;
							Set_DL(POWER_MODE, &func);
							break;

						case PM_STANDBY_MODE:

							/* Call Standby Mode Functionality */
							StandbyMode();
							break;

						default:
							pmbuf[PM_RESULT_INDEX] = FAILURE;
							break;
					}
					break;

				default:
					/* Update the result */
					pmbuf[PM_RESULT_INDEX] = FAILURE;
					break;
			}

			/* Send the response to the SDK API */
			xQueueSend( pv.pmrxq, ( void * )&pmbuf, ( TickType_t ) Q_TICKSTOWAIT );
		}
	}
}

/** @brief Start Power Management Platform service
 *
 *  This function initializes the following modules
 *     ===> Power Management DB
 *     ===> Power Management platform driver
 *     ===> Power Management software resources
 *     ===> Start the Power Management service
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 *
 */
int8_t PM_PS_Start(void)
{
	BaseType_t xReturned;

	/* Create Power Management TxQ for receiving value from the SDK API */
	pv.pmtxq = xQueueCreate( PM_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.pmtxq)
	{
		LOGERROR("Unable to initialize the Power Management TxQ\r\n");
		return PS_FAILURE;
	}

	/* Create Power Management RxQ for sending return value to the SDK API */
	pv.pmrxq = xQueueCreate( PM_RX_TX_MAX_Q_SIZE, sizeof(uint32_t*));
	if(NULL == pv.pmrxq)
	{
		LOGERROR("Unable to initialize the Power Management RxQ\r\n");
		return PS_FAILURE;
	}

	/* Start the Power Management Platform service thread */
	xReturned = xTaskCreate(PM_ServiceStart, "PM",
			512, NULL, PS_PM_TASK_PRIORITY,   &pv.pm_task);

	if (xReturned != pdPASS)
	{
		LOGERROR("Unable to initialize the Power Management Thread\r\n");
		return PS_FAILURE;
	}

	LOGERROR("Power Management Platform Service task created Successfully \r\n");

	return PS_SUCCESS;
}
#endif

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
