/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : LCD.h
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : Header File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LCD configuration structures and macros

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/
#ifndef _LCD_H_
#define	_LCD_H_

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "stdint.h"
#include "stm32h7xx.h"


/** @addtogroup BSP
  * @{
  */

/** @addtogroup Components
  * @{
  */

/** @addtogroup ST7789V
  * @{
  */

/** @defgroup ST7789V_Exported_Types ST7789V Exported Types
  * @{
  */
/**
  * @}
  */

#define LCD_REG               						((uint16_t *)0xCC000000 ) /* RS = 0 */
#define LCD_RAM               						((uint16_t *)0xCC400000) /* RS = 1 */
#define DATA_8BITS      							0x00
#define DATA_16BITS     							0x01
#define _8BITS_
//#define _16_BITS_
//#define _DATA_18_BTS_
#define _DATA_16_BTS_

/** @Color Definitions
*/
#define	RED                                         ((uint16_t)0XF800)
#define	DARK_GREEN                                  ((uint16_t)0X03E0)
#define	GREEN                                       ((uint16_t)0X07E0)
#define	DARK_CYAN                                   ((uint16_t)0X03EF)
#define	MAROON                                      ((uint16_t)0X7800)
#define	PURPLE                                      ((uint16_t)0X780F)
#define	OLIVE                                       ((uint16_t)0X7BE0)
#define	LIGHTGREY                                   ((uint16_t)0XC618)
#define	DARKGREY                                    ((uint16_t)0x7BEF)
#define	CYAN                                        ((uint16_t)0X07FF)
#define	BLUE                                        ((uint16_t)0X001F)
#define BLACK                                       ((uint16_t)0X0000)
#define WHITE                                       ((uint16_t)0XFFFF)
#define NAVY                                        ((uint16_t)0X000F)
#define MAGENTA                                     ((uint16_t)0XF81F)
#define YELLOW                                      ((uint16_t)0XFFE0)
#define ORANGE                                      ((uint16_t)0XFD20)
#define GREEN_YELLOW                                ((uint16_t)0XAFE5)
#define PINK                                        ((uint16_t)0xFC18)

#define	RGB_RED                                     ((uint32_t)0Xff0000)
#define	RGB_DARK_GREEN                              ((uint32_t)0X007d00)
#define	RGB_GREEN                                   ((uint32_t)0X00ff00)
#define	RGB_DARK_CYAN                               ((uint32_t)0X007d7b)
#define	RGB_MAROON                                  ((uint32_t)0X7b0000)
#define	RGB_PURPLE                                  ((uint32_t)0X7b007b)
#define	RGB_OLIVE                                   ((uint32_t)0X7B7b00)
#define	RGB_LIGHTGREY                               ((uint32_t)0XC6c3c6)
#define	RGB_DARKGREY                                ((uint32_t)0x7B7d7b)
#define	RGB_CYAN                                    ((uint32_t)0X00ffff)
#define	RGB_BLUE                                    ((uint32_t)0X0000ff)
#define RGB_BLACK                                   ((uint32_t)0X000000)
#define RGB_WHITE                                   ((uint32_t)0XFFFFff)
#define RGB_NAVY                                    ((uint32_t)0X00007b)
#define RGB_MAGENTA                                 ((uint32_t)0Xff00ff)
#define RGB_YELLOW                                  ((uint32_t)0XFfff00)
#define RGB_ORANGE                                  ((uint32_t)0XFfa500)
#define RGB_GREEN_YELLOW                            ((uint32_t)0XAdff29)
#define RGB_PINK                                    ((uint32_t)0xFf82c6)

#define OK 											0
#define FAIL 										1
/** @defgroup ST7789H2_Exported_Constants ST7789H2 Exported Constants
  * @{
  */
/**
  * @brief  ST7789V ID
  */
#define  ST7789V_ID    								((uint32_t)0x00858552)

/**
  * @brief  ST7789V Size
  */
#define  ST7789V_LCD_PIXEL_WIDTH    				((uint16_t)240)
#define  ST7789V_LCD_PIXEL_HEIGHT   				((uint16_t)320)

/**
 *  @brief LCD_OrientationTypeDef
 *  Possible values of Display Orientation
 */

#define ST7789V_ORIENTATION_PORTRAIT         		((uint32_t)0x00) /* Portrait orientation choice of LCD screen  */
#define ST7789V_ORIENTATION_PORTRAIT_ROT180  		((uint32_t)0x01) /* Landscape rotated 180� orientation choice of LCD screen */
#define ST7789V_ORIENTATION_LANDSCAPE        		((uint32_t)0x02) /* Landscape orientation choice of LCD screen */
#define ST7789V_ORIENTATION_LANDSCAPE_ROT180 		((uint32_t)0x03) /* Landscape rotated 180� orientation choice of LCD screen */

/**
  * @brief  ST7789V2 Registers
  */

#define INVERSION 1
#define STV7789V_NOP               					0x0000
#define ST7789V_SOFTWARE_RESET    	 				0x0001
#define ST7789V_LCD_ID             					0x0004
#define ST7789V_RDDST              					0x0009
#define ST7789V_RDDCOLMOD          					0x000c
#define ST7789V_SLEEP_IN           					0x0010
#define ST7789V_SLEEP_OUT          					0x0011
#define ST7789V_PARTIAL_DISPLAY    					0x0012
#define ST7789V_NORON              					0x0013
#define ST7789V_DISPLAY_INVERSION  					0x0020 | INVERSION
#define ST7789V_GAMSET             					0x0026
#define ST7789V_DISPLAY_ON         					0x29
#define ST7789V_WRITE_RAM          					0x002C
#define ST7789V_READ_RAM           					0x2E
#define ST7789V_CASET              					0x002A
#define ST7789V_RASET              					0x002B
//#define ST7789V_GRAM               				0x2CU   /* GRAM register */
#define ST7789V_VSCRDEF            					0x33 /* Vertical Scroll Definition */
#define ST7789V_VSCSAD             					0x37 /* Vertical Scroll Start Address of RAM */
#define ST7789V_TEARING_EFFECT     					0x35 /* Write memory continue/ 3eH read continue*/
#define ST7789V_NORMAL_DISPLAY     					0x0036
#define ST7789V_IDLE_MODE_OFF      					0x38
#define ST7789V_IDLE_MODE_ON       					0x39
#define ST7789V_COLOR_MODE         					0x003A
#define ST7789V_WBRIGHTNESS        					0x51
#define ST7789V_WCRTLDISP          					0x53
#define ST7789V_WRCACE             					0x55
#define ST7789V_WRCABCMB           					0x5E /* Write Content Adaptive Brightness Control */
#define ST7789V_RDCABCMB           					0x5F /* Read Content Adaptive Brightness Control */
#define ST7789V_RAM_CTRL           					0xb0
#define ST7789V_PORCH_CTRL         					0x00B2
#define ST7789V_FRCTRL1            					0x00B3
#define ST7789V_GATE_CTRL          					0x00B7
#define ST7789V_DGMEN              					0x00BA
#define ST7789V_VCOM_SET           					0x00BB
#define ST7789V_DISPLAY_OFF        					0xBD
#define ST7789V_LCM_CTRL           					0x00C0
#define ST7789V_VDV_VRH_EN         					0x00C2
#define ST7789V_VRH_SET            					0x00C3
#define ST7789V_VDV_SET            					0x00C4
#define SST7789V_VCOMH_OFFSET_SET   				0xC5
#define ST7789V_FR_CTRL           	 				0xC6
#define ST7789V_POWER_CTRL         					0x00D0
#define ST7789V_PV_GAMMA_CTRL      					0x00E0
#define ST7789V_NV_GAMMA_CTRL      					0x00E1
#define ST7789V_DGMLUTR            					0x00E2
#define ST7789V_DGMLUTB            					0x00E3
/**
  * @}
  */

/** @defgroup ST7789H2_Exported_Functions ST7789H2 Exported Functions
  * @{
  */
uint8_t  ST7789V_Init(void);
void     ST7789V_SetOrientation(uint32_t orientation);
uint32_t ST7789V_ReadID(void);
void     ST7789V_WriteReg(uint16_t Command, uint16_t *Parameters, uint8_t NbParameters);
uint16_t ST7789V_ReadReg(uint8_t Command);

void     ST7789V_DisplayOn(void);
void     ST7789V_DisplayOff(void);
void     ST7789V_SetCursor(uint16_t Xpos, uint16_t Ypos);
void     ST7789V_WritePixel(uint16_t Xpos, uint16_t Ypos, uint16_t RGBCode);
uint16_t ST7789V_ReadPixel(uint16_t Xpos, uint16_t Ypos);

void     ST7789V_DrawHLine(uint16_t RGBCode, uint16_t Xpos, uint16_t Ypos, uint16_t Length);
void     ST7789V_DrawVLine(uint16_t RGBCode, uint16_t Xpos, uint16_t Ypos, uint16_t Length);
void     ST7789V2_DrawBitmap(uint16_t Xpos, uint16_t Ypos, uint8_t *pbmp);
void     ST7789V2_DrawRGBImage(uint16_t Xpos, uint16_t Ypos, uint16_t Xsize, uint16_t Ysize, uint8_t *pdata);

void     ST7789V_SetDisplayWindow(uint16_t Xpos, uint16_t Ypos, uint16_t Width, uint16_t Height);


uint16_t ST7789V_GetLcdPixelWidth(void);
uint16_t ST7789V_GetLcdPixelHeight(void);
void ST7789V_backlight(uint8_t level);
/** @defgroup LCD_Driver_structure  LCD Driver structure
  * @{
  */
typedef struct
{
  uint8_t  (*Init)(void);
  uint32_t (*ReadID)(void);
  void     (*DisplayOn)(void);
  void     (*DisplayOff)(void);
  void     (*SetCursor)(uint16_t, uint16_t);
  void     (*WritePixel)(uint16_t, uint16_t, uint16_t);
  uint16_t (*ReadPixel)(uint16_t, uint16_t);

   /* Optimized operation */
  void     (*SetDisplayWindow)(uint16_t, uint16_t, uint16_t, uint16_t);
  void     (*DrawHLine)(uint16_t, uint16_t, uint16_t, uint16_t);
  void     (*DrawVLine)(uint16_t, uint16_t, uint16_t, uint16_t);

  uint16_t (*GetLcdPixelWidth)(void);
  uint16_t (*GetLcdPixelHeight)(void);
  void     (*DrawBitmap)(uint16_t, uint16_t, uint8_t*);
  void     (*DrawRGBImage)(uint16_t, uint16_t, uint16_t, uint16_t, uint8_t*);
}LCD_DrvTypeDef;
/* LCD driver structure */
extern LCD_DrvTypeDef   ST7789H2_drv;

/* LCD IO functions */
extern void     LCD_IO_Init(void);
extern void     LCD_IO_WriteMultipleData(uint16_t *pData, uint32_t Size);
extern void     LCD_WriteMultipleData(uint16_t* pData, uint32_t Size, uint32_t line);
extern void     LCD_IO_WriteReg(uint16_t Reg);
extern void     LCD_IO_WriteData(uint16_t RegValue);
extern uint16_t LCD_IO_ReadData(void);
extern void     LCD_IO_Delay(uint32_t delay);

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif	//_LCD_H_

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
