/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Configurableinput.c
   Author             : Zaid
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the Configuration Input HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Zaid

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
#include "../../Drivers/Maximatecc/inc/input_control.h"
#include "../Inc/ConfigurableInput.h"
#include "../../../Core/Maximatecc/Inc/Frequency_Timer_Init.h"
/* USER CODE END Includes */
uint8_t ci_frequency_1 = 0;
uint8_t ci_frequency_2 = 0;
uint8_t ci_frequency_3 = 0;
uint8_t ci_frequency_4 = 0;
uint8_t ci_frequency_5 = 0;

/** @brief CI Initialize
 *
 *  This function initializes the DIO Expander
 *  IO pins.
 *
 *  @param  None
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t CI_Init(I2C_HandleTypeDef *hi2c)
{
	/**
	 * CIN_VR_EN1	->	PF2
	 * CIN_PDA1		->	PF3
	 * CIN_PDB1		->	PF4
	 */
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_2, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_3, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_4, GPIO_PIN_RESET);

	/**
	 * CIN_VR_EN2	->	PG0
	 * CIN_PDA2		->	PF13
	 * CIN_PDB2		->	PF12
	 */
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_0, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_12, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_13, GPIO_PIN_RESET);

	/**
	 * CIN_VR_EN3	->	PE6
	 * CIN_PDA3		->	PI9
	 * CIN_PDB3		->	PI10
	 */
	HAL_GPIO_WritePin(GPIOE, GPIO_PIN_6, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_9, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_10, GPIO_PIN_RESET);

	/**
	 * CIN_VR_EN4	->	PH2
	 * CIN_PDA4		->	PH3
	 * CIN_PDB4		->	PH5
	 */
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_2, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_3, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_5, GPIO_PIN_RESET);

	/**
	 * CIN_VR_EN5	->	PI7
	 * CIN_PDA5		->	PI4
	 * CIN_PDB5		->	PI5
	 */
	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_7, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_4, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_5, GPIO_PIN_RESET);

	return PS_SUCCESS;

}

/** @brief Analog Input configuration
 *
 *  This function sets the Configuration Input configuration
 *  in the IO Expander Pins for the AI.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t CI_AI1(I2C_HandleTypeDef *hi2c, uint8_t type)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	switch(type)
	{
		case INPUT_FREQUENCY:
			/* CIN_VR_EN1 Pin */
	    	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_2, GPIO_PIN_SET);
			/* CIN_PDA1 Pin */
	    	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_3, GPIO_PIN_RESET);
			/* CIN_PDB1 Pin */
	    	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_4, GPIO_PIN_RESET);

			ci_frequency_1 = INPUT_FREQUENCY;
			Timer13_Init();
			break;

		case DIGITAL_STB:
		case INPUT_VOLTAGE_HIGH:
			/* CIN_VR_EN1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_2, GPIO_PIN_SET);
			/* CIN_PDA1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_3, GPIO_PIN_RESET);
			/* CIN_PDB1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_4, GPIO_PIN_SET);

			HAL_GPIO_DeInit(GPIOF, GPIO_PIN_8);
			GPIO_InitStruct.Pin = GPIO_PIN_8;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
			break;

		case DIGITAL_STG:
		case INPUT_RESISTANCE:
			/* CIN_VR_EN1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_2, GPIO_PIN_RESET);
			/* CIN_PDA1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_3, GPIO_PIN_RESET);
			/* CIN_PDB1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_4, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOF, GPIO_PIN_8);
			GPIO_InitStruct.Pin = GPIO_PIN_8;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
			break;

		case INPUT_VOLTAGE_LOW_6V:
			/* CIN_VR_EN1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_2, GPIO_PIN_SET);
			/* CIN_PDA1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_3, GPIO_PIN_SET);
			/* CIN_PDB1 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_4, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOF, GPIO_PIN_8);
			GPIO_InitStruct.Pin = GPIO_PIN_8;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
			break;
		default:
			break;
	  }

	return PS_SUCCESS;
}


/** @brief Analog Input 2 configuration
 *
 *  This function sets the Configuration Input configuration
 *  in the IO Expander Pins for the AI.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t CI_AI2(I2C_HandleTypeDef *hi2c, uint8_t type)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	switch(type)
	{
		case INPUT_FREQUENCY:
			/* CIN_VR_EN2 Pin */
	    	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_0, GPIO_PIN_SET);
			/* CIN_PDB2 Pin */
	    	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_12, GPIO_PIN_RESET);
	    	/* CIN_PDA2 Pin */
	    	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_13, GPIO_PIN_RESET);

			ci_frequency_2 = INPUT_FREQUENCY;
			Timer14_Init();
			break;

		case DIGITAL_STB:
		case INPUT_VOLTAGE_HIGH:
			/* CIN_VR_EN2 Pin */
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_0, GPIO_PIN_SET);
			/* CIN_PDB2 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_12, GPIO_PIN_SET);
	    	/* CIN_PDA2 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_13, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOF, GPIO_PIN_9);
			GPIO_InitStruct.Pin = GPIO_PIN_9;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
			break;

		case DIGITAL_STG:
		case INPUT_RESISTANCE:
			/* CIN_VR_EN2 Pin */
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_0, GPIO_PIN_RESET);
			/* CIN_PDB2 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_12, GPIO_PIN_RESET);
	    	/* CIN_PDA2 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_13, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOF, GPIO_PIN_9);
			GPIO_InitStruct.Pin = GPIO_PIN_9;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
			break;

		case INPUT_VOLTAGE_LOW_6V:
			/* CIN_VR_EN2 Pin */
			HAL_GPIO_WritePin(GPIOG, GPIO_PIN_0, GPIO_PIN_SET);
			/* CIN_PDB2 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_12, GPIO_PIN_RESET);
	    	/* CIN_PDA2 Pin */
			HAL_GPIO_WritePin(GPIOF, GPIO_PIN_13, GPIO_PIN_SET);

			HAL_GPIO_DeInit(GPIOF, GPIO_PIN_9);
			GPIO_InitStruct.Pin = GPIO_PIN_9;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);
			break;

		default:
			break;
	  }
	return PS_SUCCESS;
}

/** @brief Analog Input 3 configuration
 *
 *  This function sets the Configuration Input configuration
 *  in the IO Expander Pins for the AI.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t CI_AI3(I2C_HandleTypeDef *hi2c, uint8_t type)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	switch(type)
	{
		case INPUT_FREQUENCY:
			/* CIN_VR_EN3 Pin */
	    	HAL_GPIO_WritePin(GPIOE, GPIO_PIN_6, GPIO_PIN_SET);
			/* CIN_PDB3 Pin */
	    	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_10, GPIO_PIN_RESET);
			/* CIN_PDA3 Pin */
	    	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_9, GPIO_PIN_RESET);

			ci_frequency_3 = INPUT_FREQUENCY;
			Timer2_Init();
			break;

		case DIGITAL_STB:
		case INPUT_VOLTAGE_HIGH:
			/* CIN_VR_EN3 Pin */
			HAL_GPIO_WritePin(GPIOE, GPIO_PIN_6, GPIO_PIN_SET);
			/* CIN_PDB3 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_10, GPIO_PIN_SET);
			/* CIN_PDA3 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_9, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_1);
			GPIO_InitStruct.Pin = GPIO_PIN_1;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		case DIGITAL_STG:
		case INPUT_RESISTANCE:
			/* CIN_VR_EN3 Pin */
			HAL_GPIO_WritePin(GPIOE, GPIO_PIN_6, GPIO_PIN_RESET);
			/* CIN_PDB3 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_10, GPIO_PIN_RESET);
			/* CIN_PDA3 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_9, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_1);
			GPIO_InitStruct.Pin = GPIO_PIN_1;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		case INPUT_VOLTAGE_LOW_6V:
			/* CIN_VR_EN3 Pin */
			HAL_GPIO_WritePin(GPIOE, GPIO_PIN_6, GPIO_PIN_SET);
			/* CIN_PDB3 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_10, GPIO_PIN_RESET);
			/* CIN_PDA3 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_9, GPIO_PIN_SET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_1);
			GPIO_InitStruct.Pin = GPIO_PIN_1;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		default:
			break;
	  }
	return PS_SUCCESS;
}

/** @brief Analog Input 4 configuration
 *
 *  This function sets the Configuration Input configuration
 *  in the IO Expander Pins for the AI.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t CI_AI4(I2C_HandleTypeDef *hi2c, uint8_t type)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	switch(type)
	{
		case INPUT_FREQUENCY:
			/* CIN_VR_EN4 Pin */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_2, GPIO_PIN_SET);
			/* CIN_PDA4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_3, GPIO_PIN_RESET);
			/* CIN_PDB4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_5, GPIO_PIN_RESET);

			ci_frequency_4 = INPUT_FREQUENCY;
			Timer2_Init();
			break;

		case DIGITAL_STB:
		case INPUT_VOLTAGE_HIGH:
			/* CIN_VR_EN4 Pin */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_2, GPIO_PIN_SET);
			/* CIN_PDA4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_3, GPIO_PIN_RESET);
			/* CIN_PDB4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_5, GPIO_PIN_SET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_0);
			GPIO_InitStruct.Pin = GPIO_PIN_0;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		case DIGITAL_STG:
		case INPUT_RESISTANCE:
			/* CIN_VR_EN4 Pin */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_2, GPIO_PIN_RESET);
			/* CIN_PDA4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_3, GPIO_PIN_RESET);
			/* CIN_PDB4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_5, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_0);
			GPIO_InitStruct.Pin = GPIO_PIN_0;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		case INPUT_VOLTAGE_LOW_6V:
			/* CIN_VR_EN4 Pin */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_2, GPIO_PIN_SET);
			/* CIN_PDA4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_3, GPIO_PIN_SET);
			/* CIN_PDB4 */
	    	HAL_GPIO_WritePin(GPIOH, GPIO_PIN_5, GPIO_PIN_RESET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_0);
			GPIO_InitStruct.Pin = GPIO_PIN_0;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		default:
			break;
	  }
	return PS_SUCCESS;
}

/** @brief Analog Input 5 configuration
 *
 *  This function sets the Configuration Input configuration
 *  in the IO Expander Pins for the AI.
 *
 *  @param  None
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t CI_AI5(I2C_HandleTypeDef *hi2c, uint8_t type)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};

	switch(type)
	{
		case INPUT_FREQUENCY:
			/* CIN_VR_EN5 Pin */
	    	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_7, GPIO_PIN_SET);
			/* CIN_PDA5 */
	    	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_4, GPIO_PIN_RESET);
			/* CIN_PDB5 */
	    	HAL_GPIO_WritePin(GPIOI, GPIO_PIN_5, GPIO_PIN_RESET);

			ci_frequency_5 = INPUT_FREQUENCY;
			Timer3_Init();

			break;

		case DIGITAL_STB:
		case INPUT_VOLTAGE_HIGH:
			/* CIN_VR_EN5 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_7, GPIO_PIN_SET);
			/* CIN_PDA5 */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_4, GPIO_PIN_RESET);
			/* CIN_PDB5 */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_5, GPIO_PIN_SET);

			HAL_GPIO_DeInit(GPIOA, GPIO_PIN_7);
			GPIO_InitStruct.Pin = GPIO_PIN_7;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		case DIGITAL_STG:
		case INPUT_RESISTANCE:
			/* CIN_VR_EN5 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_7, GPIO_PIN_RESET);
			/* CIN_PDA5 */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_4, GPIO_PIN_RESET);
			/* CIN_PDB5 */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_5, GPIO_PIN_RESET);

			GPIO_InitStruct.Pin = GPIO_PIN_7;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		case INPUT_VOLTAGE_LOW_6V:
			/* CIN_VR_EN5 Pin */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_7, GPIO_PIN_SET);
			/* CIN_PDA5 */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_4, GPIO_PIN_SET);
			/* CIN_PDB5 */
			HAL_GPIO_WritePin(GPIOI, GPIO_PIN_5, GPIO_PIN_RESET);

			GPIO_InitStruct.Pin = GPIO_PIN_7;
			GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
			GPIO_InitStruct.Pull = GPIO_NOPULL;
			HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
			break;

		default:
			break;
	  }
	return PS_SUCCESS;
}

int8_t SetAI(I2C_HandleTypeDef *hi2c, uint8_t ai, INPUT_TYPE type)
{
	switch(ai)
	{
		case CONFIG_AI1://an 1
			CI_AI1(hi2c, type);
			break;
		case CONFIG_AI2://an2
			CI_AI2(hi2c, type);
			break;
		case CONFIG_AI3://an3
			CI_AI3(hi2c, type);
			break;
		case CONFIG_AI4://an4
			CI_AI4(hi2c, type);
			break;
		case CONFIG_AI5://an5
			CI_AI5(hi2c, type);
			break;
	}

	return PS_SUCCESS;
}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
