/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Fdcan.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the FDCAN HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../../Core/Maximatecc/Inc/Common.h"
#include "../../Drivers/Maximatecc/inc/Fdcan.h"

/* USER CODE END Includes */

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

extern pvt_variable pv;

FDCAN_Bit_Cfg fdcanCfg[] =
{
#ifdef __25MHz__
	{   50u,  50u,  8u, 1u }, /* 87.5% */
	{  100u,  25u,  8u, 1u }, /* 87.5% */
	{  125u,  25u,  6u, 1u }, /* 87.5% */
	{  250u,  10u,  8u, 1u }, /* 87.5% */
	{  500u,   5u,  8u, 1u }, /* 87.5% */
#endif

//#ifdef __48MHz__
	{   50u,  60u, 13u, 2u }, /* 87.5% */
	{  100u,  30u, 13u, 2u }, /* 87.5% */
	{  125u,  24u, 13u, 2u }, /* 87.5% */
	{  250u,  12u, 13u, 2u }, /* 87.5% */
	{  500u,   6u, 13u, 2u }, /* 87.5% */
	{ 1000u,   3u, 13u, 2u }, /* 87.5% */
//#endif

};


/** @brief Set the Mask Filter
 *
 *  This function does the following.
 *         -- Initialize the FDCAN
 *         -- Configure ID reception filters to Rx Buffer or Rx Fifo
 *         -- Enable the Rx
 *
 *
 *  @param  FDCAN_HandleTypeDef :- Handler
 *          IdType              :- FDCAN Id type
 *          filterType          :- FDCAN filter type
 *          filterConfig        :- Filter to be Configre whether FIFO or BUFFER
 *          filterId_1          :- Filter ID1
 *          filterID_2          :- Filter_ID2
 *          toBuffer            :- To configure the Filter for
 *
 *  @return SUCCESS
 *          FAILUER
 *
 */
uint8_t FDCAN_SetMaskFilter(FDCAN_HandleTypeDef *hdlrFdcan,
							uint32_t idType,
							uint32_t filterType,
							uint32_t filterConfig,
							uint32_t filterID_1,
							uint32_t filterID_2,
							uint8_t tobuffer)
{
	FDCAN_FilterTypeDef FilterConfig;

	if (tobuffer != TRUE)
	{
		/* Configure standard ID reception filter to Rx buffer 0 */
		FilterConfig.IdType = idType;                       // FDCAN_STANDARD_ID;
		FilterConfig.FilterIndex = 0;
		FilterConfig.FilterType = filterType;               // FDCAN_FILTER_DUAL;
		FilterConfig.FilterConfig = filterConfig;           // FDCAN_FILTER_TO_RXBUFFER;
		FilterConfig.FilterID1 = filterID_1;                // 0x111;
		FilterConfig.FilterID2 = filterID_2;                // 0x555;
		FilterConfig.RxBufferIndex = 0;
		if (HAL_FDCAN_ConfigFilter(hdlrFdcan, &FilterConfig) != HAL_OK )
		{
			return 0;
		}
	}
	else
	{
		/* Configure standard ID reception filter to Rx buffer 0 */
		FilterConfig.IdType = idType;                       //FDCAN_STANDARD_ID;
		FilterConfig.FilterIndex = 0;
		FilterConfig.FilterType = filterType;               // FDCAN_FILTER_DUAL;
		FilterConfig.FilterConfig = filterConfig;           // FDCAN_FILTER_TO_RXFIFO;
		FilterConfig.FilterID1 = filterID_1;                // 0x1111111;
		FilterConfig.FilterID2 = filterID_2;                // 0x222222;
		FilterConfig.RxBufferIndex = 0;
		if (HAL_FDCAN_ConfigFilter(hdlrFdcan, &FilterConfig) != HAL_OK )
		{
			return FALSE;
		}
	}
	return TRUE;
}

/** @brief Configure the Transmit FDCAN.
 *
 *  This function does the following.
 *         -- Initialize the FDCAN
 *         -- Configure ID reception.
 *         -- Configure the Tx Buffer message.
 *         -- Enable the Tx
 *
 *
 *  @param  FDCAN_HandleTypeDef :- Handler
 *          IdType              :- FDCAN Id type
 *          filterType          :- FDCAN filter type
 *          filterConfig        :- Filter to be Configre whether FIFO or BUFFER
 *          filterId_1          :- Filter ID1
 *          filterID_2          :- Filter_ID2
 *          toBuffer            :- To configure the Filter for
 *
 *  @return None
 *
 */
uint8_t FDCAN_ConfigureTx(FDCAN_HandleTypeDef *hdlrFdcan,
						  uint32_t ident,
						  uint32_t idType,
						  uint8_t *TxData, uint32_t TxBuffer)
{
	  FDCAN_TxHeaderTypeDef TxHeader;
	/* Configure Tx buffer message */
	TxHeader.Identifier = ident;
	TxHeader.IdType = idType;
	TxHeader.TxFrameType = FDCAN_DATA_FRAME;
	TxHeader.DataLength = FDCAN_DLC_BYTES_8;           //FDCAN_DLC_BYTES_12;
	TxHeader.ErrorStateIndicator = FDCAN_ESI_ACTIVE;
	TxHeader.BitRateSwitch = FDCAN_BRS_ON;
	TxHeader.FDFormat = FDCAN_CLASSIC_CAN;
	TxHeader.TxEventFifoControl = FDCAN_STORE_TX_EVENTS;
	TxHeader.MessageMarker = 0;

	if ( HAL_FDCAN_AddMessageToTxBuffer(hdlrFdcan, &TxHeader, TxData, TxBuffer) != HAL_OK)
	{
		return FALSE;
	}

	return TRUE;
}

/** @brief  Send the data to the FDCAN Buffer.
 *
 *  This function does the following.
 *         -- Transmit the data to the FDCAN buffer.
 *
 *
 *  @param  FDCAN_HandleTypeDef :- Handler
 *          BufferIndex         :-
 *
 *  @return SUCCESS
 *          ERROR
 */
uint8_t FDCAN_SendMessageToBuffer(FDCAN_HandleTypeDef *hdlrFdcan,
								  uint32_t bufferIndex)
{
	if (HAL_FDCAN_EnableTxBufferRequest(hdlrFdcan, bufferIndex) != HAL_OK)
	{
		return FALSE;
	}

	return TRUE;
}

/** @brief  Send the data to the FDCAN Tx FIFO.
 *
 *  This function does the following.
 *         -- Transmit the data to the FDCAN buffer.
 *
 *
 *  @param  FDCAN_HandleTypeDef   :- Handler
 *          FDCAN_TxHeaderTypeDef :- pTxHeader
 *          Data Buffer           :- pTxData
 *
 *  @return SUCCESS
 *          ERROR
 */
uint8_t FDCAN_SendMessageToTxFioQ(FDCAN_HandleTypeDef *hdlrFdcan,
								  FDCAN_TxHeaderTypeDef *pTxHeader,
								  uint8_t *pTxData)
{
	if ( HAL_FDCAN_AddMessageToTxFifoQ(hdlrFdcan, pTxHeader, pTxData) != HAL_OK)
	{
		return FALSE;
	}

	return TRUE;
}

/** @brief  Receive the data from the FDCAN.
 *
 *  This function does the following.
 *         -- Transmit the data to the FDCAN buffer.
 *
 *
 *  @param  FDCAN_HandleTypeDef   :- Handler
 *          FDCAN_TxHeaderTypeDef :- pTxHeader
 *          Data Buffer           :- pTxData
 *
 *  @return None
 *
 */
uint32_t FDCAN_RecieiveRxMessage(FDCAN_HandleTypeDef *hfdcan,
							 uint32_t RxLocation,
							 FDCAN_RxHeaderTypeDef *pRxHeader,
							 uint8_t *pRxData)
{
	uint32_t ret_status;

	/* Retrieve message from Rx FIFO 1 */
	ret_status = HAL_FDCAN_GetRxMessage(hfdcan, RxLocation, pRxHeader, pRxData);

	return ret_status;
}

/** @brief Remove the configured filter.
 *
 *  This function does the following.
 *         -- Remove the Filters
 *         -- Disable the Rx
 *
 *
 *  @param  FDCAN_HandleTypeDef :- Handler
 *          IdType              :- FDCAN Id type
 *          filterType          :- FDCAN filter type
 *          filterConfig        :- Filter to be Disable
 *          filterId_1          :- Filter ID1
 *          filterID_2          :- Filter_ID2
 *
 *
 *  @return SUCCESS
 *          FAILUER
 *
 */
uint8_t FDCAN_Remove_Filter(FDCAN_HandleTypeDef *hdlrFdcan,
							FDCAN_FilterTypeDef FilterConfig,
							uint32_t idType,
							uint8_t filterType,
							uint32_t filterConfig,
							uint32_t filterID_1,
							uint32_t filterID_2)
{
	FilterConfig.IdType = idType;                       // FDCAN_STANDARD_ID;
	FilterConfig.FilterIndex = 0;
	FilterConfig.FilterType = filterType;               // FDCAN_FILTER_DUAL;
	FilterConfig.FilterConfig = filterConfig;           // FDCAN_FILTER_DISABLE;
	FilterConfig.FilterID1 = filterID_1;                // 0x111;
	FilterConfig.FilterID2 = filterID_2;                // 0x555;
	FilterConfig.RxBufferIndex = 0;
    if (HAL_FDCAN_ConfigFilter(hdlrFdcan, &FilterConfig) != HAL_OK )
	{
		return FALSE;
	}
    return TRUE;
}

/** @brief Get the status of FDCAN Transmit/Receive.
 *
 *  This function does the following.
 *         -- Check if a transmission request is pending on the selected Tx buffer.
 *         -- Check if a new message is received in the selected Rx buffer.
 *
 *
 *  @param  FDCAN_HandleTypeDef :  hdlrFdcan pointer to an FDCAN_HandleTypeDef structure that
 *                                 contains the configuration information for the specified FDCAN.
 *          bufferIndex         :- RxBufferIndex Rx / TxBufferIndex Tx buffer index.
 *          function            :- Transfer /Recevie FDCAN
 *
 *  @return Receive:
 *          0 : No new message on RxBufferIndex.
 *          1 : New message received on RxBufferIndex.
 *
 *          Transfer :
 *          0 : No pending transmission request on TxBufferIndex.
 *          1 : Pending transmission request on TxBufferIndex.
 *
 */
uint8_t FDCAN_GetStatus(FDCAN_HandleTypeDef *hdlrFdcan, uint32_t bufferIndex, uint8_t function)
{
	uint8_t ret = 0;

	if (Transfer == function )
	{
	    ret =  HAL_FDCAN_IsTxBufferMessagePending(hdlrFdcan, bufferIndex);
	}
	else if (Receive == function)
	{
		ret = HAL_FDCAN_IsRxBufferMessageAvailable(hdlrFdcan, bufferIndex);
	}

	return ret;
}

void PS_FDCAN1_Init(uint8_t baud_idx)
{
	HAL_FDCAN_DeInit(pv.hfdcan1);

	pv.hfdcan1->Instance = FDCAN1;
	pv.hfdcan1->Init.FrameFormat = FDCAN_FRAME_FD_BRS;
	pv.hfdcan1->Init.Mode = FDCAN_MODE_NORMAL;
	pv.hfdcan1->Init.AutoRetransmission = DISABLE;
	pv.hfdcan1->Init.TransmitPause = DISABLE;
	pv.hfdcan1->Init.ProtocolException = DISABLE;
	pv.hfdcan1->Init.NominalPrescaler = fdcanCfg[baud_idx].prescaler;
	pv.hfdcan1->Init.NominalSyncJumpWidth = 8;
	pv.hfdcan1->Init.NominalTimeSeg1 = fdcanCfg[baud_idx].segment1;
	pv.hfdcan1->Init.NominalTimeSeg2 = fdcanCfg[baud_idx].segment2;
	pv.hfdcan1->Init.DataPrescaler = fdcanCfg[baud_idx].prescaler;
	pv.hfdcan1->Init.DataSyncJumpWidth = 4;
	pv.hfdcan1->Init.DataTimeSeg1 = fdcanCfg[baud_idx].segment1;
	pv.hfdcan1->Init.DataTimeSeg2 = fdcanCfg[baud_idx].segment2;
	pv.hfdcan1->Init.MessageRAMOffset = 0;
	pv.hfdcan1->Init.StdFiltersNbr = 1;
	pv.hfdcan1->Init.ExtFiltersNbr = 1;
	pv.hfdcan1->Init.RxFifo0ElmtsNbr = 1;
	pv.hfdcan1->Init.RxFifo0ElmtSize = FDCAN_DATA_BYTES_8;
	pv.hfdcan1->Init.RxFifo1ElmtsNbr = 1;
	pv.hfdcan1->Init.RxFifo1ElmtSize = FDCAN_DATA_BYTES_8;
	pv.hfdcan1->Init.RxBuffersNbr = 1;
	pv.hfdcan1->Init.RxBufferSize = FDCAN_DATA_BYTES_8;
	pv.hfdcan1->Init.TxEventsNbr = 2;
	pv.hfdcan1->Init.TxBuffersNbr = 1;
	pv.hfdcan1->Init.TxFifoQueueElmtsNbr = 2;
	pv.hfdcan1->Init.TxFifoQueueMode = FDCAN_TX_FIFO_OPERATION;
	pv.hfdcan1->Init.TxElmtSize = FDCAN_DATA_BYTES_8;
	if(HAL_FDCAN_Init(pv.hfdcan1) != HAL_OK)
	{
		Error_Handler();
	}
}

#endif	//SDK_SERVICE_FDCAN
/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
