/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : Led.c
   Author             : Adrush
   Version            : V0.0.0.1
   Origination Date   : 04/02/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the LED HAL driver functions

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             04/02/2022           Adrush

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* USER CODE BEGIN Includes */
#include "../../../Core/Maximatecc/Inc/Common.h"
#include "../../Drivers/Maximatecc/inc/LED.h"
/* USER CODE END Includes */
#define   MAX_PLATFORM_SUPPORTED_LED	2


/** @brief Led Initialize
 *
 *  This function iniitializes the LED IO Expander
 *  IO pins as Output.
 *
 *  @param  led_num [IN]: LED number
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Led_Init()
{
	/* RESET the RED LED & AMBR LED pin */
	HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_14, GPIO_PIN_RESET);

	return PS_SUCCESS;
}

/** @brief Led Off
 *
 *  This function is used for turn off LED.
 *
 *  @param  led_num [IN]: LED number
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Led_Off(uint8_t led_num)
{
	/* Set the Amber / Red LED GPIO in pin */
    if (AMBER_LED == led_num)
    {
    	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_14, GPIO_PIN_RESET);
    }
    else if(RED_LED == led_num)
    {
    	HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3, GPIO_PIN_RESET);
    }

	return PS_SUCCESS;
}

/** @brief Led On
 *
 *  This function is used for turn on LED.
 *
 *  @param  led_num [IN]: LED number
 *
 *
 *  @return 0 : PS_SUCCESS
 *          1 : PS_FAILURE
 */
int8_t Led_On(uint8_t led_num)
{
	if (!((AMBER_LED == led_num) || (RED_LED == led_num)))
		return PS_FAILURE;

	/* Set the Amber / Red LED GPIO in pin */
    if (AMBER_LED == led_num)
    {
    	HAL_GPIO_WritePin(GPIOG, GPIO_PIN_14, GPIO_PIN_SET);
    }
    else if(RED_LED == led_num)
    {
    	HAL_GPIO_WritePin(GPIOE, GPIO_PIN_3, GPIO_PIN_SET);
    }

	return PS_SUCCESS;

}

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
