/****************************************************************************
              Copyright (c) 2012 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : spi.c
   Author             : Name Last_Name
   Version            : V0.0.0.1
   Origination Date   : 04/25/2012
   Description        : C Source File.
   Environment        : Independant
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

		General File Description
-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.0              04/04/2011         Name Last_Name

   Notes:              - Initial Version
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1              31/03/2011         Name Last_Name

   Notes:              - Second Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* Includes ---------------------------------------------------------------*/
#include "../../../Core/Maximatecc/Inc/Common.h"
/* Private typedef --------------------------------------------------------*/
/* Private define ---------------------------------------------------------*/
/* Private macro ----------------------------------------------------------*/
/* Private variables ------------------------------------------------------*/

#if (SDK_SERVICE_EEPROM == PS_ENABLE)
extern pvt_variable pv;
/* Private functions declaration ------------------------------------------*/

/* PUBLIC functions definition --------------------------------------------*/
/* PRIVATE functions definition -------------------------------------------*/
/**
 * @brief Init EEPROM SPI
 *
 * @param hspi Pointer to SPI struct handler
 */
//void hspi_INIT(SPI_HandleTypeDef * hspi) {
//    hspi = hspi;
//}


uint8_t EEPROM_eraseSector(unsigned char page)
{
	unsigned int address = 128 * page;
	unsigned char twofivefive[128] = {0x00,};//clear all

	while (pv.hspi1->State != HAL_SPI_STATE_READY)
	{
		HAL_Delay(1);
	}

	HAL_StatusTypeDef spiTransmitStatus;

	sEE_WriteEnable();

		/*
	 * We gonna send commands in one packet of 3 bytes
     */
	uint8_t header[3];

	header[0] = EEPROM_WRITE;   /* Send "Write to Memory" instruction	*/
	header[1] = address >> 8; /* Send 16-bit address	*/
	header[2] = address&0xff;

	/* Select the EEPROM: Chip Select low	*/
	EEPROM_CS_LOW();

    EEPROM_SPI_SendInstruction((uint8_t*)header, 3);

	/* Make 5 attemtps to write the data	*/
    for (uint8_t i = 0; i < 5; i++)
    {
    	spiTransmitStatus = HAL_SPI_Transmit(pv.hspi1, twofivefive, 128,/* 100*/128*2);

    	if (spiTransmitStatus == HAL_BUSY)
    	{
    		HAL_Delay(5);
    	}
    	else
    	{
    		break;
        }
    }

    /* Deselect the EEPROM: Chip Select high	*/
    EEPROM_CS_HIGH();

	/* Disable the write access to the EEPROM	*/
    sEE_WriteDisable();

    if (spiTransmitStatus == HAL_ERROR)
    {
    	return EEPROM_STATUS_ERROR;
    }
    else
    {
    	return EEPROM_STATUS_COMPLETE;
    }
}

//--------------------------------------------------------------------------------------------------------------
void EEPROM_eraseAll()
{
	volatile char index;

	for(index=0; index < 128; index++)
	{
		EEPROM_eraseSector(index);
	}
}

/**
  * @brief  Writes more than one byte to the EEPROM with a single WRITE cycle
  *         (Page WRITE sequence).
  *
  * @note   The number of byte can't exceed the EEPROM page size.
  * @param  pBuffer: pointer to the buffer  containing the data to be written
  *         to the EEPROM.
  * @param  WriteAddr: EEPROM's internal address to write to.
  * @param  NumByteToWrite: number of bytes to write to the EEPROM, must be equal
  *         or less than "EEPROM_PAGESIZE" value.
  * @retval EepromOperations value: EEPROM_STATUS_COMPLETE or EEPROM_STATUS_ERROR
  */
EepromOperations EEPROM_SPI_WritePage(uint8_t* pBuffer, uint16_t WriteAddr, uint16_t NumByteToWrite)
{
	while (pv.hspi1->State != HAL_SPI_STATE_READY)
	{
		HAL_Delay(1);
	}

	HAL_StatusTypeDef spiTransmitStatus;

	sEE_WriteEnable();
	/*
	 * We gonna send commands in one packet of 3 bytes
     */
	uint8_t header[3];

	header[0] = EEPROM_WRITE;   /* Send "Write to Memory" instruction	*/
	header[1] = WriteAddr >> 8; /* Send 16-bit address	*/
	header[2] = WriteAddr;

	/* Select the EEPROM: Chip Select low	*/
	EEPROM_CS_LOW();

    EEPROM_SPI_SendInstruction((uint8_t*)header, 3);

    /* Make 5 attemtps to write the data	*/
    for (uint8_t i = 0; i < 5; i++)
    {
    	spiTransmitStatus = HAL_SPI_Transmit(pv.hspi1, pBuffer, NumByteToWrite,/* 100*/NumByteToWrite*2);

    	if (spiTransmitStatus == HAL_BUSY)
    	{
    		HAL_Delay(5);
    	}
    	else
    	{
    		break;
        }
    }

    /* Deselect the EEPROM: Chip Select high	*/
    EEPROM_CS_HIGH();

    /* Wait the end of EEPROM writing	*/
    EEPROM_SPI_WaitStandbyState();

    /* Disable the write access to the EEPROM	*/
    sEE_WriteDisable();

    if (spiTransmitStatus == HAL_ERROR)
    {
    	return EEPROM_STATUS_ERROR;
    }
    else
    {
    	return EEPROM_STATUS_COMPLETE;
    }
}

/**
  * @brief  Writes block of data to the EEPROM. In this function, the number of
  *         WRITE cycles are reduced, using Page WRITE sequence.
  *
  * @param  pBuffer: pointer to the buffer  containing the data to be written
  *         to the EEPROM.
  * @param  WriteAddr: EEPROM's internal address to write to.
  * @param  NumByteToWrite: number of bytes to write to the EEPROM.
  * @retval EepromOperations value: EEPROM_STATUS_COMPLETE or EEPROM_STATUS_ERROR
  */
EepromOperations EEPROM_SPI_WriteBuffer(uint8_t* pBuffer, uint16_t WriteAddr, uint16_t NumByteToWrite)
{
	uint16_t NumOfPage = 0, NumOfSingle = 0, Addr = 0, count = 0, temp = 0;
	uint16_t sEE_DataNum = 0;

	EepromOperations pageWriteStatus = EEPROM_STATUS_PENDING;

	Addr = WriteAddr % EEPROM_PAGESIZE;
	count = EEPROM_PAGESIZE - Addr;
	NumOfPage =  NumByteToWrite / EEPROM_PAGESIZE;
	NumOfSingle = NumByteToWrite % EEPROM_PAGESIZE;

	if (Addr == 0)
	{
		/* WriteAddr is EEPROM_PAGESIZE aligned  */
		if (NumOfPage == 0)
		{
			/* NumByteToWrite < EEPROM_PAGESIZE */
			sEE_DataNum = NumByteToWrite;
			pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

			if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
			{
				return pageWriteStatus;
			}
		}
		else
		{
			/* NumByteToWrite > EEPROM_PAGESIZE */
			while (NumOfPage--)
			{
				sEE_DataNum = EEPROM_PAGESIZE;
				pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

				if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
				{
					return pageWriteStatus;
				}

				WriteAddr +=  EEPROM_PAGESIZE;
				pBuffer += EEPROM_PAGESIZE;
			}

			sEE_DataNum = NumOfSingle;
			pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

			if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
			{
				return pageWriteStatus;
            }
		}
	}
	else
	{
		/* WriteAddr is not EEPROM_PAGESIZE aligned  */
		if (NumOfPage == 0)
		{
			/* NumByteToWrite < EEPROM_PAGESIZE */
			if (NumOfSingle > count)
			{
				/* (NumByteToWrite + WriteAddr) > EEPROM_PAGESIZE */
				temp = NumOfSingle - count;
				sEE_DataNum = count;
				pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

				if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
				{
					return pageWriteStatus;
				}

				WriteAddr +=  count;
				pBuffer += count;

				sEE_DataNum = temp;
				pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);
			}
			else
			{
				sEE_DataNum = NumByteToWrite;
				pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);
			}

			if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
			{
				return pageWriteStatus;
			}
		}
		else
		{
			/* NumByteToWrite > EEPROM_PAGESIZE */
			NumByteToWrite -= count;
			NumOfPage =  NumByteToWrite / EEPROM_PAGESIZE;
			NumOfSingle = NumByteToWrite % EEPROM_PAGESIZE;

			sEE_DataNum = count;

			pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

			if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
			{
				return pageWriteStatus;
			}

			WriteAddr +=  count;
			pBuffer += count;

			while (NumOfPage--)
			{
				sEE_DataNum = EEPROM_PAGESIZE;

				pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

				if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
				{
					return pageWriteStatus;
				}

				WriteAddr +=  EEPROM_PAGESIZE;
				pBuffer += EEPROM_PAGESIZE;
			}

			if (NumOfSingle != 0)
			{
				sEE_DataNum = NumOfSingle;

				pageWriteStatus = EEPROM_SPI_WritePage(pBuffer, WriteAddr, sEE_DataNum);

				if (pageWriteStatus != EEPROM_STATUS_COMPLETE)
				{
					return pageWriteStatus;
                }
			}
		}
	}

	return EEPROM_STATUS_COMPLETE;
}

/**
  * @brief  Reads a block of data from the EEPROM.
  *
  * @param  pBuffer: pointer to the buffer that receives the data read from the EEPROM.
  * @param  ReadAddr: EEPROM's internal address to read from.
  * @param  NumByteToRead: number of bytes to read from the EEPROM.
  * @retval None
  */
EepromOperations EEPROM_SPI_ReadBuffer(uint8_t* pBuffer, uint16_t ReadAddr, uint16_t NumByteToRead)
{
	while (pv.hspi1->State != HAL_SPI_STATE_READY)
	{
		HAL_Delay(1);
	}

	/*
	 * We gonna send all commands in one packet of 3 bytes
	 */
	uint8_t header[3];

	header[0] = EEPROM_READ;    /* Send "Read from Memory" instruction	*/
	header[1] = ReadAddr >> 8;  /* Send 16-bit address	*/
	header[2] = ReadAddr;

	/* Select the EEPROM: Chip Select low	*/
	EEPROM_CS_LOW();

	/* Send WriteAddr address byte to read from */
	EEPROM_SPI_SendInstruction((uint8_t*)header, 3);

	while (HAL_SPI_Receive(pv.hspi1, (uint8_t*)pBuffer, NumByteToRead, NumByteToRead*2) == HAL_BUSY)
	{
		HAL_Delay(1);
    };

	/* Deselect the EEPROM: Chip Select high	*/
    EEPROM_CS_HIGH();

    return EEPROM_STATUS_COMPLETE;
}

/**
  * @brief  Sends a byte through the SPI interface and return the byte received
  *         from the SPI bus.
  *
  * @param  byte: byte to send.
  * @retval The value of the received byte.
  */
uint8_t EEPROM_SendByte(uint8_t byte)
{
	uint8_t answerByte;

	/* Loop while DR register in not empty */
	while (pv.hspi1->State == HAL_SPI_STATE_RESET)
	{
        HAL_Delay(1);
    }

    /* Send byte through the SPI peripheral */
    if (HAL_SPI_Transmit(pv.hspi1, &byte, 1, 200) != HAL_OK)
    {
    	Error_Handler();
    }

    /* Wait to receive a byte */
    while (pv.hspi1->State == HAL_SPI_STATE_RESET)
    {
        HAL_Delay(1);
    }

    /* Return the byte read from the SPI bus */
    if (HAL_SPI_Receive(pv.hspi1, &answerByte, 1, 200) != HAL_OK)
    {
    	Error_Handler();
    }

    return (uint8_t)answerByte;
}

/**
  * @brief  Enables the write access to the EEPROM.
  *
  * @param  None
  * @retval None
  */
void sEE_WriteEnable(void)
{
	/* Select the EEPROM: Chip Select low	*/
	EEPROM_CS_LOW();

	uint8_t command[1] = { EEPROM_WREN };
	/* Send "Write Enable" instruction */
	EEPROM_SPI_SendInstruction((uint8_t*)command, 1);

	/* Deselect the EEPROM: Chip Select high	*/
	EEPROM_CS_HIGH();
}

/**
  * @brief  Disables the write access to the EEPROM.
  *
  * @param  None
  * @retval None
  */
void sEE_WriteDisable(void)
{
	/* Select the EEPROM: Chip Select low	*/
    EEPROM_CS_LOW();

    uint8_t command[1] = { EEPROM_WRDI };

    /* Send "Write Disable" instruction */
    EEPROM_SPI_SendInstruction((uint8_t*)command, 1);

    /* Deselect the EEPROM: Chip Select high	*/
    EEPROM_CS_HIGH();
}

/**
  * @brief  Write new value in EEPROM Status Register.
  *
  * @param  regval : new value of register
  * @retval None
  */
void sEE_WriteStatusRegister(uint8_t regval)
{
	uint8_t command[2];

    command[0] = EEPROM_WRSR;
    command[1] = regval;

    /* Enable the write access to the EEPROM	*/
    sEE_WriteEnable();

    /* Select the EEPROM: Chip Select low	*/
    EEPROM_CS_LOW();

    /* Send "Write Status Register" instruction
     * and Regval in one packet
     */
    EEPROM_SPI_SendInstruction((uint8_t*)command, 2);

    /* Deselect the EEPROM: Chip Select high	*/
    EEPROM_CS_HIGH();

    sEE_WriteDisable();
}


/**
  * @brief  Polls the status of the Write In Progress (WIP) flag in the EEPROM's
  *         status register and loop until write operation has completed.
  *
  * @param  None
  * @retval None
  */
uint8_t EEPROM_SPI_WaitStandbyState(void)
{
	uint8_t sEEstatus[1] = { 0x00 };
    uint8_t command[1] = { EEPROM_RDSR };

    /* Select the EEPROM: Chip Select low	*/
    EEPROM_CS_LOW();

    /* Send "Read Status Register" instruction	*/
    EEPROM_SPI_SendInstruction((uint8_t*)command, 1);

    /* Loop as long as the memory is busy with a write cycle	*/
    do
    {
    	while (HAL_SPI_Receive(pv.hspi1, (uint8_t*)sEEstatus, 1, 200) == HAL_BUSY)
    	{
            HAL_Delay(1);
        };

        HAL_Delay(1);

    } while ((sEEstatus[0] & EEPROM_WIP_FLAG) == SET); /* Write in progress	*/

    /* Deselect the EEPROM: Chip Select high	*/
    EEPROM_CS_HIGH();

    return 0;
}

/**
 * @brief Low level function to send header data to EEPROM
 *
 * @param instruction array of bytes to send
 * @param size        data size in bytes
 */
void EEPROM_SPI_SendInstruction(uint8_t *instruction, uint8_t size)
{
	while (pv.hspi1->State == HAL_SPI_STATE_RESET)
	{
		HAL_Delay(1);
    }

    if (HAL_SPI_Transmit(pv.hspi1, (uint8_t*)instruction, (uint16_t)size, 200) != HAL_OK)
    {
    	Error_Handler();
    }
}
#endif	//SDK_SERVICE_EEPROM
/**************** (C) COPYRIGHT 2012 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
